@@ -14,7 +14,7 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 // sbar.c -- status bar code
@@ -25,38 +25,46 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "draw.h"
 #include "protocol.h"
 #include "quakedef.h"
+#include "qpic.h"
 #include "sbar.h"
 #include "screen.h"
 #include "vid.h"
 #include "wad.h"
 
-int sb_updates;			// if >= vid.numpages, no update needed
+/* if >= vid.numpages, no update needed */
+int sb_updates;
 
-#define STAT_MINUS		10	// num frame for '-' stats digit
-qpic_t *sb_nums[2][11];
-qpic_t *sb_colon, *sb_slash;
-qpic_t *sb_ibar;
-qpic_t *sb_sbar;
-qpic_t *sb_scorebar;
+/* num frame for '-' stats digit */
+#define STAT_MINUS 10
 
-qpic_t *sb_weapons[7][8];	// 0 is active, 1 is owned, 2-5 are flashes
-qpic_t *sb_ammo[4];
-qpic_t *sb_sigil[4];
-qpic_t *sb_armor[3];
-qpic_t *sb_items[32];
+const qpic8_t *sb_nums[2][11];
+const qpic8_t *sb_colon, *sb_slash;
+const qpic8_t *sb_ibar;
+const qpic8_t *sb_sbar;
+const qpic8_t *sb_scorebar;
 
-qpic_t *sb_faces[7][2];		// 0 is gibbed, 1 is dead, 2-6 are alive
+/* 0 is active, 1 is owned, 2-5 are flashes */
+const qpic8_t *sb_weapons[7][8];
 
-							// 0 is static, 1 is temporary animation
-qpic_t *sb_face_invis;
-qpic_t *sb_face_quad;
-qpic_t *sb_face_invuln;
-qpic_t *sb_face_invis_invuln;
+const qpic8_t *sb_ammo[4];
+const qpic8_t *sb_sigil[4];
+const qpic8_t *sb_armor[3];
+const qpic8_t *sb_items[32];
+
+/* [0][] is gibbed, [1][] is dead, [2-6][] are alive */
+/* [][0] is static, [][1] is temporary animation */
+const qpic8_t *sb_faces[7][2];
+
+const qpic8_t *sb_face_invis;
+const qpic8_t *sb_face_quad;
+const qpic8_t *sb_face_invuln;
+const qpic8_t *sb_face_invis_invuln;
 
 qboolean sb_showscores;
 qboolean sb_showteamscores;
 
-int sb_lines;			// scan lines to draw
+int sb_lines;                   // scan lines to draw
+int sb_lines_hidden;            // scan lines obscured totally by the status bar
 
 void Sbar_DeathmatchOverlay(int start);
 void Sbar_TeamOverlay(void);
@@ -137,13 +145,17 @@ Sbar_Changed(void)
     sb_updates = 0;		// update next frame
 }
 
-/*
-===============
-Sbar_Init
-===============
-*/
 void
-Sbar_Init(void)
+Sbar_AddCommands()
+{
+    Cmd_AddCommand("+showscores", Sbar_ShowScores);
+    Cmd_AddCommand("-showscores", Sbar_DontShowScores);
+    Cmd_AddCommand("+showteamscores", Sbar_ShowTeamScores);
+    Cmd_AddCommand("-showteamscores", Sbar_DontShowTeamScores);
+}
+
+static void
+Sbar_InitPics(void)
 {
     int i;
 
@@ -221,17 +233,21 @@ Sbar_Init(void)
     sb_face_invis_invuln = Draw_PicFromWad("face_inv2");
     sb_face_quad = Draw_PicFromWad("face_quad");
 
-    Cmd_AddCommand("+showscores", Sbar_ShowScores);
-    Cmd_AddCommand("-showscores", Sbar_DontShowScores);
-
-    Cmd_AddCommand("+showteamscores", Sbar_ShowTeamScores);
-    Cmd_AddCommand("-showteamscores", Sbar_DontShowTeamScores);
-
     sb_sbar = Draw_PicFromWad("sbar");
     sb_ibar = Draw_PicFromWad("ibar");
     sb_scorebar = Draw_PicFromWad("scorebar");
 }
 
+/*
+===============
+Sbar_Init
+===============
+*/
+void
+Sbar_Init(void)
+{
+    Sbar_InitPics();
+}
 
 //=============================================================================
 
@@ -242,11 +258,10 @@ Sbar_Init(void)
 Sbar_DrawPic
 =============
 */
-void
-Sbar_DrawPic(int x, int y, qpic_t *pic)
+static void
+Sbar_DrawPic(int x, int y, const qpic8_t *pic)
 {
-    Draw_Pic(x /* + ((vid.width - 320)>>1) */ ,
-	     y + (vid.height - SBAR_HEIGHT), pic);
+    Draw_PicAlpha(x, y + (scr_scaled_height - SBAR_HEIGHT), pic, scr_sbaralpha.value);
 }
 
 /*
@@ -256,12 +271,10 @@ Sbar_DrawSubPic
 JACK: Draws a portion of the picture in the status bar.
 */
 
-void
-Sbar_DrawSubPic(int x, int y, qpic_t *pic, int srcx, int srcy, int width,
-		int height)
+static void
+Sbar_DrawSubPic(int x, int y, const qpic8_t *pic, int srcx, int srcy, int width, int height)
 {
-    Draw_SubPic(x, y + (vid.height - SBAR_HEIGHT), pic, srcx, srcy, width,
-		height);
+    Draw_SubPicAlpha(x, y + (scr_scaled_height - SBAR_HEIGHT), pic, srcx, srcy, width, height, scr_sbaralpha.value);
 }
 
 
@@ -270,11 +283,10 @@ Sbar_DrawSubPic(int x, int y, qpic_t *pic, int srcx, int srcy, int width,
 Sbar_DrawTransPic
 =============
 */
-void
-Sbar_DrawTransPic(int x, int y, qpic_t *pic)
+static void
+Sbar_DrawTransPic(int x, int y, const qpic8_t *pic)
 {
-    Draw_TransPic(x /*+ ((vid.width - 320)>>1) */ ,
-		  y + (vid.height - SBAR_HEIGHT), pic);
+    Draw_TransPicAlpha(x, y + (scr_scaled_height - SBAR_HEIGHT), pic, TRANSPARENT_COLOR, scr_sbaralpha.value);
 }
 
 /*
@@ -284,11 +296,10 @@ Sbar_DrawCharacter
 Draws one solid graphics character
 ================
 */
-void
+static void
 Sbar_DrawCharacter(int x, int y, int num)
 {
-    Draw_Character(x /*+ ((vid.width - 320)>>1) */  + 4,
-		   y + vid.height - SBAR_HEIGHT, num);
+    Draw_CharacterAlpha(x + 4, y + scr_scaled_height - SBAR_HEIGHT, num, scr_sbaralpha.value);
 }
 
 /*
@@ -296,11 +307,10 @@ Sbar_DrawCharacter(int x, int y, int num)
 Sbar_DrawString
 ================
 */
-void
-Sbar_DrawString(int x, int y, char *str)
+static void
+Sbar_DrawString(int x, int y, const char *str)
 {
-    Draw_String(x /*+ ((vid.width - 320)>>1) */ ,
-		y + vid.height - SBAR_HEIGHT, str);
+    Draw_StringAlpha(x, y + scr_scaled_height - SBAR_HEIGHT, str, scr_sbaralpha.value);
 }
 
 /*
@@ -308,7 +318,7 @@ Sbar_DrawString(int x, int y, char *str)
 Sbar_itoa
 =============
 */
-int
+static int
 Sbar_itoa(int num, char *buf)
 {
     char *str;
@@ -342,7 +352,7 @@ Sbar_itoa(int num, char *buf)
 Sbar_DrawNum
 =============
 */
-void
+static void
 Sbar_DrawNum(int x, int y, int num, int digits, int color)
 {
     char str[12];
@@ -482,13 +492,13 @@ Sbar_SortTeams(void)
 	    }
 }
 
-int
+static int
 Sbar_ColorForMap(int m)
 {
-    m = (m < 0) ? 0 : ((m > 13) ? 13 : m);
+    m = qclamp(m, 0, 13) * 16;
 
-    m *= 16;
-    return m < 128 ? m + 8 : m + 8;
+    //return m < 128 ? m + 8 : m + 8;
+    return m + 8;
 }
 
 
@@ -510,7 +520,7 @@ Sbar_SoloScoreboard(void)
     seconds = cl.time - 60 * minutes;
     tens = seconds / 10;
     units = seconds - 10 * tens;
-    sprintf(str, "Time :%3i:%i%i", minutes, tens, units);
+    qsnprintf(str, sizeof(str), "Time :%3i:%i%i", minutes, tens, units);
     Sbar_DrawString(184, 4, str);
 }
 
@@ -540,9 +550,7 @@ Sbar_DrawInventory(void)
     for (i = 0; i < 7; i++) {
 	if (cl.stats[STAT_ITEMS] & (IT_SHOTGUN << i)) {
 	    time = cl.item_gettime[i];
-	    flashon = (int)((cl.time - time) * 10);
-	    if (flashon < 0)
-		flashon = 0;
+	    flashon = qmax((int)((cl.time - time) * 10), 0);
 	    if (flashon >= 10) {
 		if (cl.stats[STAT_ACTIVEWEAPON] == (IT_SHOTGUN << i))
 		    flashon = 1;
@@ -552,8 +560,8 @@ Sbar_DrawInventory(void)
 		flashon = (flashon % 5) + 2;
 
 	    if (headsup) {
-		if (i || vid.height > 200)
-		    Sbar_DrawSubPic((hudswap) ? 0 : (vid.width - 24),
+		if (i || scr_scaled_height > 200)
+		    Sbar_DrawSubPic((hudswap) ? 0 : (scr_scaled_width - 24),
 				    -68 - (7 - i) * 16,
 				    sb_weapons[flashon][i], 0, 0, 24, 16);
 
@@ -568,31 +576,22 @@ Sbar_DrawInventory(void)
 
 // ammo counts
     for (i = 0; i < 4; i++) {
-	sprintf(num, "%3i", cl.stats[STAT_SHELLS + i]);
+	qsnprintf(num, sizeof(num), "%3i", cl.stats[STAT_SHELLS + i]);
 	if (headsup) {
-//                      Sbar_DrawSubPic(3, -24, sb_ibar, 3, 0, 42,11);
-	    Sbar_DrawSubPic((hudswap) ? 0 : (vid.width - 42),
-			    -24 - (4 - i) * 11, sb_ibar, 3 + (i * 48), 0, 42,
-			    11);
+	    Sbar_DrawSubPic((hudswap) ? 0 : (scr_scaled_width - 42), -24 - (4 - i) * 11, sb_ibar, 3 + (i * 48), 0, 42, 11);
 	    if (num[0] != ' ')
-		Sbar_DrawCharacter((hudswap) ? 3 : (vid.width - 39),
-				   -24 - (4 - i) * 11, 18 + num[0] - '0');
+		Sbar_DrawCharacter((hudswap) ? 3 : (scr_scaled_width - 39), -24 - (4 - i) * 11, 18 + num[0] - '0');
 	    if (num[1] != ' ')
-		Sbar_DrawCharacter((hudswap) ? 11 : (vid.width - 31),
-				   -24 - (4 - i) * 11, 18 + num[1] - '0');
+		Sbar_DrawCharacter((hudswap) ? 11 : (scr_scaled_width - 31), -24 - (4 - i) * 11, 18 + num[1] - '0');
 	    if (num[2] != ' ')
-		Sbar_DrawCharacter((hudswap) ? 19 : (vid.width - 23),
-				   -24 - (4 - i) * 11, 18 + num[2] - '0');
+		Sbar_DrawCharacter((hudswap) ? 19 : (scr_scaled_width - 23), -24 - (4 - i) * 11, 18 + num[2] - '0');
 	} else {
 	    if (num[0] != ' ')
-		Sbar_DrawCharacter((6 * i + 1) * 8 - 2, -24,
-				   18 + num[0] - '0');
+		Sbar_DrawCharacter((6 * i + 1) * 8 - 2, -24, 18 + num[0] - '0');
 	    if (num[1] != ' ')
-		Sbar_DrawCharacter((6 * i + 2) * 8 - 2, -24,
-				   18 + num[1] - '0');
+		Sbar_DrawCharacter((6 * i + 2) * 8 - 2, -24, 18 + num[1] - '0');
 	    if (num[2] != ' ')
-		Sbar_DrawCharacter((6 * i + 3) * 8 - 2, -24,
-				   18 + num[2] - '0');
+		Sbar_DrawCharacter((6 * i + 3) * 8 - 2, -24, 18 + num[2] - '0');
 	}
     }
 
@@ -643,8 +642,7 @@ Sbar_DrawFrags(void)
     l = scoreboardlines <= 4 ? scoreboardlines : 4;
 
     x = 23;
-//      xofs = (vid.width - 320)>>1;
-    y = vid.height - SBAR_HEIGHT - 23;
+    y = scr_scaled_height - SBAR_HEIGHT - 23;
 
     for (i = 0; i < l; i++) {
 	k = fragsort[i];
@@ -655,22 +653,14 @@ Sbar_DrawFrags(void)
 	    continue;
 
 	// draw background
-	top = s->topcolor;
-	bottom = s->bottomcolor;
-	top = (top < 0) ? 0 : ((top > 13) ? 13 : top);
-	bottom = (bottom < 0) ? 0 : ((bottom > 13) ? 13 : bottom);
-
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
-
-//              Draw_Fill (xofs + x*8 + 10, y, 28, 4, top);
-//              Draw_Fill (xofs + x*8 + 10, y+4, 28, 3, bottom);
-	Draw_Fill(x * 8 + 10, y, 28, 4, top);
-	Draw_Fill(x * 8 + 10, y + 4, 28, 3, bottom);
+	top = Sbar_ColorForMap(s->topcolor);
+	bottom = Sbar_ColorForMap(s->bottomcolor);
+	Draw_FillAlpha(x * 8 + 10, y, 28, 4, top, scr_sbaralpha.value);
+	Draw_FillAlpha(x * 8 + 10, y + 4, 28, 3, bottom, scr_sbaralpha.value);
 
 	// draw number
 	f = s->frags;
-	sprintf(num, "%3i", f);
+	qsnprintf(num, sizeof(num), "%3i", f);
 
 	Sbar_DrawCharacter((x + 1) * 8, -24, num[0]);
 	Sbar_DrawCharacter((x + 2) * 8, -24, num[1]);
@@ -744,8 +734,7 @@ Sbar_DrawNormal(void)
 	Sbar_DrawNum(24, 0, 666, 3, 1);
 	Sbar_DrawPic(0, 0, draw_disc);
     } else {
-	Sbar_DrawNum(24, 0, cl.stats[STAT_ARMOR], 3,
-		     cl.stats[STAT_ARMOR] <= 25);
+	Sbar_DrawNum(24, 0, cl.stats[STAT_ARMOR], 3, cl.stats[STAT_ARMOR] <= 25);
 	if (cl.stats[STAT_ITEMS] & IT_ARMOR3)
 	    Sbar_DrawPic(0, 0, sb_armor[2]);
 	else if (cl.stats[STAT_ITEMS] & IT_ARMOR2)
@@ -758,8 +747,7 @@ Sbar_DrawNormal(void)
     Sbar_DrawFace();
 
 // health
-    Sbar_DrawNum(136, 0, cl.stats[STAT_HEALTH], 3,
-		 cl.stats[STAT_HEALTH] <= 25);
+    Sbar_DrawNum(136, 0, cl.stats[STAT_HEALTH], 3, cl.stats[STAT_HEALTH] <= 25);
 
 // ammo icon
     if (cl.stats[STAT_ITEMS] & IT_SHELLS)
@@ -786,7 +774,7 @@ Sbar_Draw(void)
     char st[512];
 
     headsup = !(cl_sbar.value || scr_viewsize.value < 100);
-    if ((sb_updates >= vid.numpages) && !headsup)
+    if (vid.numpages && (sb_updates >= vid.numpages) && !headsup)
 	return;
 
     if (scr_con_current == vid.height)
@@ -801,7 +789,7 @@ Sbar_Draw(void)
     if (sb_lines > 24) {
 	if (!cl.spectator || autocam == CAM_TRACK)
 	    Sbar_DrawInventory();
-	if (!headsup || vid.width < 512)
+	if (!headsup || scr_scaled_width < 512)
 	    Sbar_DrawFrags();
     }
 // main area
@@ -810,17 +798,14 @@ Sbar_Draw(void)
 	    if (autocam != CAM_TRACK) {
 		Sbar_DrawPic(0, 0, sb_scorebar);
 		Sbar_DrawString(160 - 7 * 8, 4, "SPECTATOR MODE");
-		Sbar_DrawString(160 - 14 * 8 + 4, 12,
-				"Press [ATTACK] for AutoCamera");
+		Sbar_DrawString(160 - 14 * 8 + 4, 12, "Press [ATTACK] for AutoCamera");
 	    } else {
 		if (sb_showscores || cl.stats[STAT_HEALTH] <= 0)
 		    Sbar_SoloScoreboard();
 		else
 		    Sbar_DrawNormal();
 
-//                                      Sbar_DrawString (160-14*8+4,4, "SPECTATOR MODE - TRACK CAMERA");
-		sprintf(st, "Tracking %-.13s, [JUMP] for next",
-			cl.players[spec_track].name);
+		qsnprintf(st, sizeof(st), "Tracking %-.13s, [JUMP] for next", cl.players[spec_track].name);
 		Sbar_DrawString(0, -8, st);
 	    }
 	} else if (sb_showscores || cl.stats[STAT_HEALTH] <= 0)
@@ -831,8 +816,7 @@ Sbar_Draw(void)
 // main screen deathmatch rankings
     // if we're dead show team scores in team games
     if (cl.stats[STAT_HEALTH] <= 0 && !cl.spectator)
-	if (atoi(Info_ValueForKey(cl.serverinfo, "teamplay")) > 0 &&
-	    !sb_showscores)
+	if (atoi(Info_ValueForKey(cl.serverinfo, "teamplay")) > 0 && !sb_showscores)
 	    Sbar_TeamOverlay();
 	else
 	    Sbar_DeathmatchOverlay(0);
@@ -845,20 +829,8 @@ Sbar_Draw(void)
     if (sb_showscores || sb_showteamscores || cl.stats[STAT_HEALTH] <= 0)
 	sb_updates = 0;
     // clear unused areas in gl
-#if 0
-    {
-	int x = (vid.width - 320) >> 1;
-
-	// left
-	if (x > 0) {
-	    Draw_TileClear(0, vid.height - sb_lines, x, sb_lines);
-	    Draw_TileClear(x + 320, vid.height - sb_lines,
-			   vid.width - x + 320, sb_lines);
-	}
-    }
-#endif
-    if (vid.width > 320 && !headsup)
-	Draw_TileClear(320, vid.height - sb_lines, vid.width - 320, sb_lines);
+    if (scr_scaled_width > 320 && !headsup)
+	Draw_TileClearScaled(320, scr_scaled_height - sb_lines_hidden, scr_scaled_width - 320, sb_lines_hidden);
 #endif
 
     if (sb_lines > 0)
@@ -893,7 +865,7 @@ Sbar_IntermissionNumber(int x, int y, int num, int digits, int color)
 	else
 	    frame = *ptr - '0';
 
-	Draw_TransPic(x, y, sb_nums[color][frame]);
+	Draw_TransPic(x, y, sb_nums[color][frame], TRANSPARENT_COLOR);
 	x += 24;
 	ptr++;
     }
@@ -910,8 +882,8 @@ added by Zoid
 void
 Sbar_TeamOverlay(void)
 {
-    qpic_t *pic;
-    int i, k, l;
+    const qpic8_t *pic;
+    int i, k;
     int x, y;
     char num[12];
     int teamplay;
@@ -935,20 +907,17 @@ Sbar_TeamOverlay(void)
 
     y = 24;
     x = 36;
-    Draw_String(x, y, "low/avg/high team total players");
+    Draw_StringAlpha(x, y, "low/avg/high team total players", scr_sbaralpha.value);
     y += 8;
 //      Draw_String(x, y, "------------ ---- ----- -------");
-    Draw_String(x, y,
-		"\x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1e\x1e\x1f");
+    Draw_StringAlpha(x, y, "\x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1e\x1e\x1f", scr_sbaralpha.value);
     y += 8;
 
 // sort the teams
     Sbar_SortTeams();
 
 // draw the text
-    l = scoreboardlines;
-
-    for (i = 0; i < scoreboardteams && y <= vid.height - 10; i++) {
+    for (i = 0; i < scoreboardteams && y <= scr_scaled_height - 10; i++) {
 	k = teamsort[i];
 	tm = teams + k;
 
@@ -966,26 +935,24 @@ Sbar_TeamOverlay(void)
 	if (pavg < 0 || pavg > 999)
 	    pavg = 999;
 
-	sprintf(num, "%3i/%3i/%3i", plow, pavg, phigh);
-	Draw_String(x, y, num);
+	qsnprintf(num, sizeof(num), "%3i/%3i/%3i", plow, pavg, phigh);
+	Draw_StringAlpha(x, y, num, scr_sbaralpha.value);
 
 	// draw team
-	team[4] = 0;
-	strncpy(team, tm->team, 4);
-	Draw_String(x + 104, y, team);
+	qstrncpy(team, tm->team, sizeof(team));
+	Draw_StringAlpha(x + 104, y, team, scr_sbaralpha.value);
 
 	// draw total
-	sprintf(num, "%5i", tm->frags);
-	Draw_String(x + 104 + 40, y, num);
+	qsnprintf(num, sizeof(num), "%5i", tm->frags);
+	Draw_StringAlpha(x + 104 + 40, y, num, scr_sbaralpha.value);
 
 	// draw players
-	sprintf(num, "%5i", tm->players);
-	Draw_String(x + 104 + 88, y, num);
+	qsnprintf(num, sizeof(num), "%5i", tm->players);
+	Draw_StringAlpha(x + 104 + 88, y, num, scr_sbaralpha.value);
 
-	if (!strncmp(Info_ValueForKey(cl.players[cl.playernum].userinfo,
-				      "team"), tm->team, 16)) {
-	    Draw_Character(x + 104 - 8, y, 16);
-	    Draw_Character(x + 104 + 32, y, 17);
+	if (!strncmp(Info_ValueForKey(cl.players[cl.playernum].userinfo, "team"), tm->team, 16)) {
+	    Draw_CharacterAlpha(x + 104 - 8, y, 16, scr_sbaralpha.value);
+	    Draw_CharacterAlpha(x + 104 + 32, y, 17, scr_sbaralpha.value);
 	}
 
 	y += 8;
@@ -1004,7 +971,7 @@ ping time frags name
 void
 Sbar_DeathmatchOverlay(int start)
 {
-    qpic_t *pic;
+    const qpic8_t *pic;
     int i, k, l;
     int top, bottom;
     int x, y, f;
@@ -1024,7 +991,7 @@ Sbar_DeathmatchOverlay(int start)
     if (realtime - cl.last_ping_request > 2) {
 	cl.last_ping_request = realtime;
 	MSG_WriteByte(&cls.netchan.message, clc_stringcmd);
-	SZ_Print(&cls.netchan.message, "pings");
+	MSG_WriteString(&cls.netchan.message, "pings");
     }
 
     teamplay = atoi(Info_ValueForKey(cl.serverinfo, "teamplay"));
@@ -1049,24 +1016,22 @@ Sbar_DeathmatchOverlay(int start)
     if (teamplay) {
 	x = 4;
 //                            0    40 64   104   152  192
-	Draw_String(x, y, "ping pl time frags team name");
+	Draw_StringAlpha(x, y, "ping pl time frags team name", scr_sbaralpha.value);
 	y += 8;
 //              Draw_String ( x , y, "---- -- ---- ----- ---- ----------------");
-	Draw_String(x, y,
-		    "\x1d\x1e\x1e\x1f \x1d\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f");
+	Draw_StringAlpha(x, y, "\x1d\x1e\x1e\x1f \x1d\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f", scr_sbaralpha.value);
 	y += 8;
     } else {
 	x = 16;
 //                            0    40 64   104   152
-	Draw_String(x, y, "ping pl time frags name");
+	Draw_StringAlpha(x, y, "ping pl time frags name", scr_sbaralpha.value);
 	y += 8;
 //              Draw_String ( x , y, "---- -- ---- ----- ----------------");
-	Draw_String(x, y,
-		    "\x1d\x1e\x1e\x1f \x1d\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f");
+	Draw_StringAlpha(x, y, "\x1d\x1e\x1e\x1f \x1d\x1f \x1d\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1f \x1d\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1e\x1f", scr_sbaralpha.value);
 	y += 8;
     }
 
-    for (i = 0; i < l && y <= vid.height - 10; i++) {
+    for (i = 0; i < l && y <= scr_scaled_height - 10; i++) {
 	k = fragsort[i];
 	s = &cl.players[k];
 	if (!s->name[0])
@@ -1076,24 +1041,24 @@ Sbar_DeathmatchOverlay(int start)
 	p = s->ping;
 	if (p < 0 || p > 999)
 	    p = 999;
-	sprintf(num, "%4i", p);
-	Draw_String(x, y, num);
+	qsnprintf(num, sizeof(num), "%4i", p);
+	Draw_StringAlpha(x, y, num, scr_sbaralpha.value);
 
 	// draw pl
 	p = s->pl;
-	sprintf(num, "%3i", p);
+	qsnprintf(num, sizeof(num), "%3i", p);
 	if (p > 25)
-	    Draw_Alt_String(x + 32, y, num);
+	    Draw_Alt_StringAlpha(x + 32, y, num, scr_sbaralpha.value);
 	else
-	    Draw_String(x + 32, y, num);
+	    Draw_StringAlpha(x + 32, y, num, scr_sbaralpha.value);
 
 	if (s->spectator) {
-	    Draw_String(x + 40, y, "(spectator)");
+	    Draw_StringAlpha(x + 40, y, "(spectator)", scr_sbaralpha.value);
 	    // draw name
 	    if (teamplay)
-		Draw_String(x + 152 + 40, y, s->name);
+		Draw_StringAlpha(x + 152 + 40, y, s->name, scr_sbaralpha.value);
 	    else
-		Draw_String(x + 152, y, s->name);
+		Draw_StringAlpha(x + 152, y, s->name, scr_sbaralpha.value);
 	    y += skip;
 	    continue;
 	}
@@ -1103,49 +1068,47 @@ Sbar_DeathmatchOverlay(int start)
 	else
 	    total = realtime - s->entertime;
 	minutes = (int)total / 60;
-	sprintf(num, "%4i", minutes);
-	Draw_String(x + 64, y, num);
+	qsnprintf(num, sizeof(num), "%4i", minutes);
+	Draw_StringAlpha(x + 64, y, num, scr_sbaralpha.value);
 
 	// draw background
-	top = s->topcolor;
-	bottom = s->bottomcolor;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
+	top = Sbar_ColorForMap(s->topcolor);
+	bottom = Sbar_ColorForMap(s->bottomcolor);
 
 	if (largegame)
-	    Draw_Fill(x + 104, y + 1, 40, 3, top);
+	    Draw_FillAlpha(x + 104, y + 1, 40, 3, top, scr_sbaralpha.value);
 	else
-	    Draw_Fill(x + 104, y, 40, 4, top);
-	Draw_Fill(x + 104, y + 4, 40, 4, bottom);
+	    Draw_FillAlpha(x + 104, y, 40, 4, top, scr_sbaralpha.value);
+	Draw_FillAlpha(x + 104, y + 4, 40, 4, bottom, scr_sbaralpha.value);
 
 	// draw number
 	f = s->frags;
-	sprintf(num, "%3i", f);
+	qsnprintf(num, sizeof(num), "%3i", f);
 
-	Draw_Character(x + 112, y, num[0]);
-	Draw_Character(x + 120, y, num[1]);
-	Draw_Character(x + 128, y, num[2]);
+	Draw_CharacterAlpha(x + 112, y, num[0], scr_sbaralpha.value);
+	Draw_CharacterAlpha(x + 120, y, num[1], scr_sbaralpha.value);
+	Draw_CharacterAlpha(x + 128, y, num[2], scr_sbaralpha.value);
 
 	if (k == cl.playernum) {
-	    Draw_Character(x + 104, y, 16);
-	    Draw_Character(x + 136, y, 17);
+	    Draw_CharacterAlpha(x + 104, y, 16, scr_sbaralpha.value);
+	    Draw_CharacterAlpha(x + 136, y, 17, scr_sbaralpha.value);
 	}
 	// team
 	if (teamplay) {
 	    team[4] = 0;
 	    strncpy(team, Info_ValueForKey(s->userinfo, "team"), 4);
-	    Draw_String(x + 152, y, team);
+	    Draw_StringAlpha(x + 152, y, team, scr_sbaralpha.value);
 	}
 	// draw name
 	if (teamplay)
-	    Draw_String(x + 152 + 40, y, s->name);
+	    Draw_StringAlpha(x + 152 + 40, y, s->name, scr_sbaralpha.value);
 	else
-	    Draw_String(x + 152, y, s->name);
+	    Draw_StringAlpha(x + 152, y, s->name, scr_sbaralpha.value);
 
 	y += skip;
     }
 
-    if (y >= vid.height - 10)	// we ran over the screen size, squish
+    if (y >= scr_scaled_height - 10)	// we ran over the screen size, squish
 	largegame = true;
 }
 
@@ -1161,133 +1124,117 @@ displayed to right of status bar if there's room
 void
 Sbar_MiniDeathmatchOverlay(void)
 {
-    int i, k;
-    int top, bottom;
-    int x, y, f;
-    char num[12];
-    player_info_t *s;
+    int x, y, line, numlines, top, bottom;
     int teamplay;
-    char team[5];
-    int numlines;
-    char name[16 + 1];
-    team_t *tm;
 
-    if (vid.width < 512 || !sb_lines)
-	return;			// not enuff room
+    /* Don't bother if not enough room */
+    if (scr_scaled_width < 512 || !sb_lines)
+	return;
 
     teamplay = atoi(Info_ValueForKey(cl.serverinfo, "teamplay"));
 
     scr_copyeverything = 1;
     scr_fullupdate = 0;
 
-// scores
     Sbar_SortFrags(false);
-    if (vid.width >= 640)
+    if (scr_scaled_width >= 640)
 	Sbar_SortTeams();
-
     if (!scoreboardlines)
-	return;			// no one there?
-
-// draw the text
-    y = vid.height - sb_lines - 1;
-    numlines = sb_lines / 8;
-    if (numlines < 3)
-	return;			// not enough room
-
-    // find us
-    for (i = 0; i < scoreboardlines; i++)
-	if (fragsort[i] == cl.playernum)
-	    break;
-
-    if (i == scoreboardlines)	// we're not there, we are probably a spectator, just display top
-	i = 0;
-    else			// figure out start
-	i = i - numlines / 2;
-
-    if (i > scoreboardlines - numlines)
-	i = scoreboardlines - numlines;
-    if (i < 0)
-	i = 0;
-
-    x = 324;
-
-    for ( /* */ ; i < scoreboardlines && y < vid.height - 8 + 1; i++) {
-	k = fragsort[i];
-	s = &cl.players[k];
-	if (!s->name[0])
-	    continue;
-
-	// draw ping
-	top = s->topcolor;
-	bottom = s->bottomcolor;
-	top = Sbar_ColorForMap(top);
-	bottom = Sbar_ColorForMap(bottom);
-
-	Draw_Fill(x, y + 1, 40, 3, top);
-	Draw_Fill(x, y + 4, 40, 4, bottom);
-
-	// draw number
-	f = s->frags;
-	sprintf(num, "%3i", f);
-
-	Draw_Character(x + 8, y, num[0]);
-	Draw_Character(x + 16, y, num[1]);
-	Draw_Character(x + 24, y, num[2]);
-
-	if (k == cl.playernum) {
-	    Draw_Character(x, y, 16);
-	    Draw_Character(x + 32, y, 17);
-	}
-	// team
-	if (teamplay) {
-	    team[4] = 0;
-	    strncpy(team, Info_ValueForKey(s->userinfo, "team"), 4);
-	    Draw_String(x + 48, y, team);
-	}
-	// draw name
-	name[16] = 0;
-	strncpy(name, s->name, 16);
-	if (teamplay)
-	    Draw_String(x + 48 + 40, y, name);
-	else
-	    Draw_String(x + 48, y, name);
-	y += 8;
-    }
-
-    // draw teams if room
-    if (vid.width < 640 || !teamplay)
 	return;
 
-    // draw seperator
-    x += 208;
-    for (y = vid.height - sb_lines; y < vid.height - 6; y += 2)
-	Draw_Character(x, y, 14);
+    /* Check for space to draw the text */
+    y = scr_scaled_height - sb_lines - 1;
+    numlines = sb_lines / 8;
+    if (numlines < 3)
+	return;
 
-    x += 16;
+    /* Find client in the scoreboard, if not there (spectator) display top */
+    for (line = 0; line < scoreboardlines; line++)
+	if (fragsort[line] == cl.playernum)
+	    break;
+    if (line == scoreboardlines)
+	line = 0;
 
-    y = vid.height - sb_lines;
-    for (i = 0; i < scoreboardteams && y <= vid.height; i++) {
-	k = teamsort[i];
-	tm = teams + k;
+    /* Put the client in the centre of the displayed lines */
+    line = qclamp(line - numlines / 2, 0, scoreboardlines - numlines);
 
-	// draw pings
-	team[4] = 0;
-	strncpy(team, tm->team, 4);
-	Draw_String(x, y, team);
+    x = 324;
+    while (line < scoreboardlines && y < scr_scaled_height - 8 + 1) {
+	const int playernum = fragsort[line++];
+	const player_info_t *player = &cl.players[playernum];
+	if (!player->name[0])
+	    continue;
 
-	// draw total
-	sprintf(num, "%5i", tm->frags);
-	Draw_String(x + 40, y, num);
+	/* draw background */
+	top = Sbar_ColorForMap(player->topcolor);
+	bottom = Sbar_ColorForMap(player->bottomcolor);
+	Draw_FillAlpha(x, y + 1, 40, 3, top, scr_sbaralpha.value);
+	Draw_FillAlpha(x, y + 4, 40, 4, bottom, scr_sbaralpha.value);
 
-	if (!strncmp(Info_ValueForKey(cl.players[cl.playernum].userinfo,
-				      "team"), tm->team, 16)) {
-	    Draw_Character(x - 8, y, 16);
-	    Draw_Character(x + 32, y, 17);
+	/* draw frags */
+	char frags[4];
+	qsnprintf(frags, sizeof(frags), "%3d", player->frags);
+	Draw_CharacterAlpha(x + 8, y, frags[0], scr_sbaralpha.value);
+	Draw_CharacterAlpha(x + 16, y, frags[1], scr_sbaralpha.value);
+	Draw_CharacterAlpha(x + 24, y, frags[2], scr_sbaralpha.value);
+	if (playernum == cl.playernum) {
+	    Draw_CharacterAlpha(x, y, 16, scr_sbaralpha.value);
+	    Draw_CharacterAlpha(x + 32, y, 17, scr_sbaralpha.value);
 	}
 
+	/* draw team name */
+	if (teamplay) {
+	    const char *playerteam = Info_ValueForKey(player->userinfo, "team");
+	    char short_team[5];
+	    qsnprintf(short_team, sizeof(short_team), "%-4s", playerteam);
+	    Draw_StringAlpha(x + 48, y, short_team, scr_sbaralpha.value);
+	}
+
+	/* draw name */
+	char name[17];
+	qsnprintf(name, sizeof(name), "%-16s", player->name);
+	if (teamplay)
+	    Draw_StringAlpha(x + 48 + 40, y, name, scr_sbaralpha.value);
+	else
+	    Draw_StringAlpha(x + 48, y, name, scr_sbaralpha.value);
+
 	y += 8;
     }
 
+    /* draw teams if room */
+    if (scr_scaled_width < 640 || !teamplay)
+	return;
+
+    /* draw seperator */
+    x += 208;
+    for (y = scr_scaled_height - sb_lines; y < scr_scaled_height - 6; y += 2)
+	Draw_CharacterAlpha(x, y, 14, scr_sbaralpha.value);
+
+    const player_info_t *player = &cl.players[cl.playernum];
+    const char *playerteam = Info_ValueForKey(player->userinfo, "team");
+
+    x += 16;
+    y = scr_scaled_height - sb_lines;
+    line = 0;
+    while (line < scoreboardteams && y <= scr_scaled_height) {
+	const team_t *team = teams + teamsort[line++];
+
+	/* draw teamname */
+	char short_team[5];
+	qsnprintf(short_team, sizeof(short_team), "%-4s", team->team);
+	Draw_StringAlpha(x, y, short_team, scr_sbaralpha.value);
+
+	/* draw frags */
+	char frags[4];
+	qsnprintf(frags, sizeof(frags), "%3d", team->frags);
+	Draw_StringAlpha(x + 40, y, frags, scr_sbaralpha.value);
+	if (!strncmp(team->team, playerteam, 16)) {
+	    Draw_CharacterAlpha(x - 8, y, 16, scr_sbaralpha.value);
+	    Draw_CharacterAlpha(x + 32, y, 17, scr_sbaralpha.value);
+	}
+
+	y += 8;
+    }
 }
 
 
@@ -1320,10 +1267,10 @@ Sbar_FinaleOverlay
 void
 Sbar_FinaleOverlay(void)
 {
-    qpic_t *pic;
+    const qpic8_t *pic;
 
     scr_copyeverything = 1;
 
     pic = Draw_CachePic("gfx/finale.lmp");
-    Draw_TransPic((vid.width - pic->width) / 2, 16, pic);
+    Draw_TransPic((scr_scaled_width - pic->width) / 2, 16, pic, TRANSPARENT_COLOR);
 }
