@@ -14,10 +14,11 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
+#include "buildinfo.h"
 #include "cmd.h"
 #include "console.h"
 #include "model.h"
@@ -39,20 +40,18 @@ int host_hunklevel;
 
 netadr_t master_adr[MAX_MASTERS];	// address of group servers
 
-client_t *host_client;		// current client
-
 cvar_t sv_mintic = { "sv_mintic", "0.03" };	// bound the size of the
 cvar_t sv_maxtic = { "sv_maxtic", "0.1" };	// physics time tic
 
 cvar_t developer = { "developer", "0" };	// show extra messages
 
-cvar_t timeout = { "timeout", "65" };		// seconds without any message
-cvar_t zombietime = { "zombietime", "2" };	// seconds to sink messages
+static cvar_t timeout = { "timeout", "65" };	// seconds without any message
+static cvar_t zombietime = { "zombietime", "2" }; // seconds to sink messages
 						// after disconnect
 
-cvar_t rcon_password = { "rcon_password", "" };	// password for remote server commands
-cvar_t password = { "password", "" };	// password for entering the game
-cvar_t spectator_password = { "spectator_password", "" };	// password for entering as a sepctator
+static cvar_t rcon_password = { "rcon_password", "" };	// for remote commands
+static cvar_t password = { "password", "" };	// for entering the game
+static cvar_t spectator_password = { "spectator_password", "" }; // for entering as a spectator
 
 cvar_t allow_download = { "allow_download", "1" };
 cvar_t allow_download_skins = { "allow_download_skins", "1" };
@@ -61,32 +60,28 @@ cvar_t allow_download_sounds = { "allow_download_sounds", "1" };
 cvar_t allow_download_maps = { "allow_download_maps", "1" };
 
 cvar_t sv_highchars = { "sv_highchars", "1" };
-
 cvar_t sv_phs = { "sv_phs", "1" };
-
 cvar_t pausable = { "pausable", "1" };
 
-
 //
 // game rules mirrored in svs.info
 //
-cvar_t fraglimit = { "fraglimit", "0", false, true };
-cvar_t timelimit = { "timelimit", "0", false, true };
-cvar_t teamplay = { "teamplay", "0", false, true };
-cvar_t samelevel = { "samelevel", "0", false, true };
-cvar_t maxclients = { "maxclients", "8", false, true };
-cvar_t maxspectators = { "maxspectators", "8", false, true };
-cvar_t deathmatch = { "deathmatch", "1", false, true };	// 0, 1, or 2
-cvar_t spawn = { "spawn", "0", false, true };
-cvar_t watervis = { "watervis", "0", false, true };
-
-cvar_t hostname = { "hostname", "unnamed", false, true };
+cvar_t fraglimit = { "fraglimit", "0", .info = true };
+cvar_t timelimit = { "timelimit", "0", .info = true };
+cvar_t teamplay = { "teamplay", "0", .info = true };
+cvar_t samelevel = { "samelevel", "0", .info = true };
+cvar_t maxclients = { "maxclients", "8", .info = true };
+cvar_t maxspectators = { "maxspectators", "8", .info = true };
+cvar_t deathmatch = { "deathmatch", "1", .info = true };	// 0, 1, or 2
+cvar_t spawn = { "spawn", "0", .info = true };
+cvar_t watervis = { "watervis", "0", .info = true };
+cvar_t hostname = { "hostname", "unnamed", .info = true };
 
 FILE *sv_logfile;
 FILE *sv_fraglogfile;
 
-void SV_AcceptClient(netadr_t adr, int userid, char *userinfo);
-void Master_Shutdown(void);
+static void Master_Heartbeat(void);
+static void Master_Shutdown(void);
 
 //============================================================================
 
@@ -127,7 +122,7 @@ then exits
 ================
 */
 void
-SV_Error(char *error, ...)
+SV_Error(const char *fmt, ...)
 {
     va_list argptr;
     static char string[MAX_PRINTMSG];
@@ -138,8 +133,8 @@ SV_Error(char *error, ...)
 
     inerror = true;
 
-    va_start(argptr, error);
-    vsnprintf(string, sizeof(string), error, argptr);
+    va_start(argptr, fmt);
+    qvsnprintf(string, sizeof(string), fmt, argptr);
     va_end(argptr);
 
     Con_Printf("%s: %s\n", __func__, string);
@@ -162,7 +157,7 @@ to totally exit after returning from this function.
 ==================
 */
 void
-SV_FinalMessage(char *message)
+SV_FinalMessage(const char *message)
 {
     int i;
     client_t *cl;
@@ -302,6 +297,11 @@ SV_FullClientUpdate(client_t *client, sizebuf_t *buf)
     MSG_WriteByte(buf, i);
     MSG_WriteFloat(buf, realtime - client->connection_started);
 
+    /*
+     * FIXME - this is safe becuase client->userinfo is also MAX_INFO_STRING
+     * bytes, but this is just ugly. Maintain the prefixed keys in a separate
+     * buffer?
+     */
     strcpy(info, client->userinfo);
     Info_RemovePrefixedKeys(info, '_');	// server passwords, etc
 
@@ -346,7 +346,7 @@ Responds with all the info that qplug or qspy can see
 This message can be up to around 5k with worst case string lengths.
 ================
 */
-void
+static void
 SVC_Status(void)
 {
     int i;
@@ -355,7 +355,7 @@ SVC_Status(void)
     int top, bottom;
 
     Cmd_TokenizeString("status");
-    SV_BeginRedirect(RD_PACKET);
+    SV_BeginRedirect(RD_PACKET, NULL);
     Con_Printf("%s\n", svs.info);
     for (i = 0; i < MAX_CLIENTS; i++) {
 	cl = &svs.clients[i];
@@ -382,16 +382,16 @@ SV_CheckLog
 
 ===================
 */
-#define	LOG_HIGHWATER	4096
-#define	LOG_FLUSH		10*60
-void
+#define LOG_HIGHWATER 4096
+#define LOG_FLUSH 10*60
+static void
 SV_CheckLog(void)
 {
     sizebuf_t *sz;
 
     sz = &svs.log[svs.logsequence & 1];
 
-    // bump sequence if allmost full, or ten minutes have passed and
+    // bump sequence if almost full, or ten minutes have passed and
     // there is something still sitting there
     if (sz->cursize > LOG_HIGHWATER
 	|| (realtime - svs.logtime > LOG_FLUSH && sz->cursize)) {
@@ -415,7 +415,7 @@ the same as the current sequence, an A2A_NACK will be returned
 instead of the data.
 ================
 */
-void
+static void
 SVC_Log(void)
 {
     int seq;
@@ -426,7 +426,8 @@ SVC_Log(void)
     else
 	seq = -1;
 
-    if (seq == svs.logsequence - 1 || !sv_fraglogfile) {	// they allready have this data, or we aren't logging frags
+    /* they already have this data, or we aren't logging frags */
+    if (seq == svs.logsequence - 1 || !sv_fraglogfile) {
 	data[0] = A2A_NACK;
 	NET_SendPacket(1, data, net_from);
 	return;
@@ -435,7 +436,7 @@ SVC_Log(void)
     Con_DPrintf("sending log %i to %s\n", svs.logsequence - 1,
 		NET_AdrToString(net_from));
 
-    sprintf(data, "stdlog %i\n", svs.logsequence - 1);
+    qsnprintf(data, sizeof(data), "stdlog %i\n", svs.logsequence - 1);
     strcat(data, (char *)svs.log_buf[((svs.logsequence - 1) & 1)]);
 
     NET_SendPacket(strlen(data) + 1, data, net_from);
@@ -448,7 +449,7 @@ SVC_Ping
 Just responds with an acknowledgement
 ================
 */
-void
+static void
 SVC_Ping(void)
 {
     char data;
@@ -469,7 +470,7 @@ flood the server with invalid connection IPs.  With a
 challenge, they must give a valid IP address.
 =================
 */
-void
+static void
 SVC_GetChallenge(void)
 {
     int i;
@@ -508,15 +509,13 @@ SVC_DirectConnect
 A connection request that did not come from the master
 ==================
 */
-void
+static void
 SVC_DirectConnect(void)
 {
-    char userinfo[1024];
     static int userid;
     netadr_t adr;
     int i;
-    client_t *cl, *newcl;
-    client_t temp;
+    client_t *cl, *newcl, temp;
     edict_t *ent;
     int edictnum;
     char *s;
@@ -525,6 +524,7 @@ SVC_DirectConnect(void)
     int qport;
     int version;
     int challenge;
+    char userinfo[sizeof(newcl->userinfo)];
 
     version = atoi(Cmd_Argv(1));
     if (version != PROTOCOL_VERSION) {
@@ -535,12 +535,8 @@ SVC_DirectConnect(void)
     }
 
     qport = atoi(Cmd_Argv(2));
-
     challenge = atoi(Cmd_Argv(3));
-
-    // note an extra byte is needed to replace spectator key
-    strncpy(userinfo, Cmd_Argv(4), sizeof(userinfo) - 2);
-    userinfo[sizeof(userinfo) - 2] = 0;
+    qstrncpy(userinfo, Cmd_Argv(4), sizeof(userinfo));
 
     // see if the challenge is valid
     for (i = 0; i < MAX_CHALLENGES; i++) {
@@ -596,20 +592,21 @@ SVC_DirectConnect(void)
 
     newcl->userid = userid;
 
-    // works properly
     if (!sv_highchars.value) {
-	byte *p, *q;
+        /* Custom strcpy which drops any chars outside ascii range */
+        byte *src = (byte *)userinfo;
+        byte *dst = (byte *)newcl->userinfo;
+        for (i = 0; *src && i < sizeof(newcl->userinfo) - 1; src++, i++) {
+            if (*src > 31 && *src <= 127) {
+                *dst++ = *src;
+            }
+        }
+        *dst = 0;
+    } else {
+        qstrncpy(newcl->userinfo, userinfo, sizeof(newcl->userinfo));
+    }
 
-	for (p = (byte *)newcl->userinfo, q = (byte *)userinfo;
-	     *q
-	     && p < (byte *)newcl->userinfo + sizeof(newcl->userinfo) - 1;
-	     q++)
-	    if (*q > 31 && *q <= 127)
-		*p++ = *q;
-    } else
-	strncpy(newcl->userinfo, userinfo, sizeof(newcl->userinfo) - 1);
-
-    // if there is allready a slot for this ip, drop it
+    // if there is already a slot for this ip, drop it
     for (i = 0, cl = svs.clients; i < MAX_CLIENTS; i++, cl++) {
 	if (cl->state == cs_free)
 	    continue;
@@ -731,7 +728,7 @@ Shift down the remaining args
 Redirect all printfs
 ===============
 */
-void
+static void
 SVC_RemoteCommand(void)
 {
     int i;
@@ -740,12 +737,12 @@ SVC_RemoteCommand(void)
     if (!Rcon_Validate()) {
 	Con_Printf("Bad rcon from %s:\n%s\n", NET_AdrToString(net_from),
 		   net_message.data + 4);
-	SV_BeginRedirect(RD_PACKET);
+	SV_BeginRedirect(RD_PACKET, NULL);
 	Con_Printf("Bad rcon_password.\n");
     } else {
 	Con_Printf("Rcon from %s:\n%s\n", NET_AdrToString(net_from),
 		   net_message.data + 4);
-	SV_BeginRedirect(RD_PACKET);
+	SV_BeginRedirect(RD_PACKET, NULL);
 	remaining[0] = 0;
 
 	for (i = 2; i < Cmd_Argc(); i++) {
@@ -768,44 +765,38 @@ Clients that are in the game can still send
 connectionless packets.
 =================
 */
-void
+static void
 SV_ConnectionlessPacket(void)
 {
-    char *s;
-    char *c;
+    const char *cmdstring;
+    const char *cmd;
 
     MSG_BeginReading();
     MSG_ReadLong();		// skip the -1 marker
 
-    s = MSG_ReadStringLine();
-    Cmd_TokenizeString(s);
-    c = Cmd_Argv(0);
+    cmdstring = MSG_ReadStringLine();
+    Cmd_TokenizeString(cmdstring);
+    cmd = Cmd_Argv(0);
 
-    if (!strcmp(c, "ping")
-	|| (c[0] == A2A_PING && (c[1] == 0 || c[1] == '\n'))) {
+    if (!strcmp(cmd, "ping"))
 	SVC_Ping();
-	return;
-    }
-    if (c[0] == A2A_ACK && (c[1] == 0 || c[1] == '\n')) {
+    else if (cmd[0] == A2A_PING && (cmd[1] == 0 || cmd[1] == '\n'))
+	SVC_Ping();
+    else if (cmd[0] == A2A_ACK && (cmd[1] == 0 || cmd[1] == '\n'))
 	Con_Printf("A2A_ACK from %s\n", NET_AdrToString(net_from));
-	return;
-    } else if (!strcmp(c, "status")) {
+    else if (!strcmp(cmd, "status"))
 	SVC_Status();
-	return;
-    } else if (!strcmp(c, "log")) {
+    else if (!strcmp(cmd, "log"))
 	SVC_Log();
-	return;
-    } else if (!strcmp(c, "connect")) {
+    else if (!strcmp(cmd, "connect"))
 	SVC_DirectConnect();
-	return;
-    } else if (!strcmp(c, "getchallenge")) {
+    else if (!strcmp(cmd, "getchallenge"))
 	SVC_GetChallenge();
-	return;
-    } else if (!strcmp(c, "rcon"))
+    else if (!strcmp(cmd, "rcon"))
 	SVC_RemoteCommand();
     else
 	Con_Printf("bad connectionless packet from %s:\n%s\n",
-		   NET_AdrToString(net_from), s);
+		   NET_AdrToString(net_from), cmdstring);
 }
 
 /*
@@ -845,61 +836,88 @@ local network.
 ==============================================================================
 */
 
-
 typedef struct {
-    unsigned mask;
-    unsigned compare;
+    union {
+	unsigned l;
+	byte b[4];
+    } mask;
+    union {
+	unsigned l;
+	byte b[4];
+    } addr;
 } ipfilter_t;
 
 #define	MAX_IPFILTERS	1024
 
-ipfilter_t ipfilters[MAX_IPFILTERS];
-int numipfilters;
+static ipfilter_t ipfilters[MAX_IPFILTERS];
+static int numipfilters;
 
-cvar_t filterban = { "filterban", "1" };
+static cvar_t filterban = { "filterban", "1" };
 
 /*
 =================
 StringToFilter
 =================
 */
-qboolean
-StringToFilter(char *s, ipfilter_t * f)
+static qboolean
+StringToFilter(const char *in, ipfilter_t *f)
 {
-    char num[128];
-    int i, j;
-    byte b[4];
-    byte m[4];
+    char num[4];
+    int i, j, b;
+    const char *s = in;
 
     for (i = 0; i < 4; i++) {
-	b[i] = 0;
-	m[i] = 0;
+	f->addr.b[i] = 0;
+	f->mask.b[i] = 0;
     }
 
     for (i = 0; i < 4; i++) {
-	if (*s < '0' || *s > '9') {
-	    Con_Printf("Bad filter address: %s\n", s);
-	    return false;
-	}
+	if (*s < '0' || *s > '9')
+	    goto badaddr;
 
 	j = 0;
-	while (*s >= '0' && *s <= '9') {
+	while (*s >= '0' && *s <= '9' && j < 4)
 	    num[j++] = *s++;
-	}
 	num[j] = 0;
-	b[i] = atoi(num);
-	if (b[i] != 0)
-	    m[i] = 255;
+	b = atoi(num);
+	if (b > 0xff) {
+	    s -= j;
+	    goto badaddr;
+	}
+	f->addr.b[i] = b;
+	f->mask.b[i] = 0xff;
 
-	if (!*s)
+	if (!*s || *s == '/')
 	    break;
-	s++;
+	if (i < 3) {
+	    if (*s != '.')
+		goto badaddr;
+	    s++;
+	    if (!*s)
+		break;
+	}
     }
 
-    f->mask = *(unsigned *)m;
-    f->compare = *(unsigned *)b;
-
+    /* If the IP or prefix looks ok, user can specify a mask */
+    if (*s) {
+	if (*s != '/')
+	    goto badaddr;
+	s++;
+	j = 0;
+	while (*s >= '0' && *s <= '9' && j < 3)
+	    num[j++] = *s++;
+	num[j] = 0;
+	b = atoi(num);
+	if (b > 32 || *s)
+	    goto badaddr;
+	f->mask.l = BigLong(0xffffffff << (32 - b));
+    }
     return true;
+
+ badaddr:
+    Con_Printf("Bad filter address: %s\n", in);
+    Con_Printf("                    %*s\n", (int)(s - in) + 1, "^");
+    return false;
 }
 
 /*
@@ -907,13 +925,13 @@ StringToFilter(char *s, ipfilter_t * f)
 SV_AddIP_f
 =================
 */
-void
+static void
 SV_AddIP_f(void)
 {
     int i;
 
     for (i = 0; i < numipfilters; i++)
-	if (ipfilters[i].compare == 0xffffffff)
+	if (ipfilters[i].addr.l == 0xffffffff)
 	    break;		// free spot
     if (i == numipfilters) {
 	if (numipfilters == MAX_IPFILTERS) {
@@ -924,7 +942,7 @@ SV_AddIP_f(void)
     }
 
     if (!StringToFilter(Cmd_Argv(1), &ipfilters[i]))
-	ipfilters[i].compare = 0xffffffff;
+	ipfilters[i].addr.l = 0xffffffff;
 }
 
 /*
@@ -932,7 +950,7 @@ SV_AddIP_f(void)
 SV_RemoveIP_f
 =================
 */
-void
+static void
 SV_RemoveIP_f(void)
 {
     ipfilter_t f;
@@ -940,14 +958,16 @@ SV_RemoveIP_f(void)
 
     if (!StringToFilter(Cmd_Argv(1), &f))
 	return;
-    for (i = 0; i < numipfilters; i++)
-	if (ipfilters[i].mask == f.mask && ipfilters[i].compare == f.compare) {
+    for (i = 0; i < numipfilters; i++) {
+	const ipfilter_t *tmp = &ipfilters[i];
+	if (tmp->mask.l == f.mask.l && tmp->addr.l == f.addr.l) {
 	    for (j = i + 1; j < numipfilters; j++)
 		ipfilters[j - 1] = ipfilters[j];
 	    numipfilters--;
 	    Con_Printf("Removed.\n");
 	    return;
 	}
+    }
     Con_Printf("Didn't find %s.\n", Cmd_Argv(1));
 }
 
@@ -956,16 +976,21 @@ SV_RemoveIP_f(void)
 SV_ListIP_f
 =================
 */
-void
+static void
 SV_ListIP_f(void)
 {
-    int i;
-    byte b[4];
+    int i, j, mask;
 
     Con_Printf("Filter list:\n");
     for (i = 0; i < numipfilters; i++) {
-	*(unsigned *)b = ipfilters[i].compare;
-	Con_Printf("%3i.%3i.%3i.%3i\n", b[0], b[1], b[2], b[3]);
+	const byte *b = ipfilters[i].addr.b;
+	const unsigned m = ipfilters[i].mask.l;
+	mask = 0;
+	for (j = 0; j < 32; j++) {
+	    if ((m >> j) & 0x1)
+		mask++;
+	}
+	Con_Printf("%3i.%3i.%3i.%3i / %2i\n", b[0], b[1], b[2], b[3], mask);
     }
 }
 
@@ -974,15 +999,14 @@ SV_ListIP_f(void)
 SV_WriteIP_f
 =================
 */
-void
+static void
 SV_WriteIP_f(void)
 {
     FILE *f;
     char name[MAX_OSPATH];
-    byte b[4];
     int i;
 
-    sprintf(name, "%s/listip.cfg", com_gamedir);
+    qsnprintf(name, sizeof(name), "%s/listip.cfg", com_gamedir);
 
     Con_Printf("Writing %s.\n", name);
 
@@ -993,7 +1017,7 @@ SV_WriteIP_f(void)
     }
 
     for (i = 0; i < numipfilters; i++) {
-	*(unsigned *)b = ipfilters[i].compare;
+	const byte *b = ipfilters[i].addr.b;
 	fprintf(f, "addip %i.%i.%i.%i\n", b[0], b[1], b[2], b[3]);
     }
 
@@ -1005,7 +1029,7 @@ SV_WriteIP_f(void)
 SV_SendBan
 =================
 */
-void
+static void
 SV_SendBan(void)
 {
     char data[128];
@@ -1023,16 +1047,21 @@ SV_SendBan(void)
 SV_FilterPacket
 =================
 */
-qboolean
+static qboolean
 SV_FilterPacket(void)
 {
     int i;
     unsigned in;
 
-    in = *(unsigned *)net_from.ip;
+    /* FIXME - be smarter about this */
+    in  = net_from.ip.b[0] << 24;
+    in |= net_from.ip.b[1] << 16;
+    in |= net_from.ip.b[2] << 8;
+    in |= net_from.ip.b[3];
+    in = BigLong(in);
 
     for (i = 0; i < numipfilters; i++)
-	if ((in & ipfilters[i].mask) == ipfilters[i].compare)
+	if ((in & ipfilters[i].mask.l) == ipfilters[i].addr.l)
 	    return filterban.value;
 
     return !filterban.value;
@@ -1045,15 +1074,13 @@ SV_FilterPacket(void)
 SV_ReadPackets
 =================
 */
-void
+static void
 SV_ReadPackets(void)
 {
     int i;
     client_t *cl;
-    qboolean good;
     int qport;
 
-    good = false;
     while (NET_GetPacket()) {
 	if (SV_FilterPacket()) {
 	    SV_SendBan();	// tell them we aren't listening...
@@ -1086,7 +1113,6 @@ SV_ReadPackets(void)
 	    if (Netchan_Process(&cl->netchan)) {
 		/* this is a valid, sequenced packet, so process it */
 		svs.stats.packets++;
-		good = true;
 		cl->send_message = true;	/* reply at end of frame */
 		if (cl->state != cs_zombie)
 		    SV_ExecuteClientMessage(cl);
@@ -1115,7 +1141,7 @@ for a few seconds to make sure any final reliable message gets resent
 if necessary
 ==================
 */
-void
+static void
 SV_CheckTimeouts(void)
 {
     int i;
@@ -1154,7 +1180,7 @@ SV_GetConsoleCommands
 Add them exactly as if they had been typed at the console
 ===================
 */
-void
+static void
 SV_GetConsoleCommands(void)
 {
     char *cmd;
@@ -1163,7 +1189,7 @@ SV_GetConsoleCommands(void)
 	cmd = Sys_ConsoleInput();
 	if (!cmd)
 	    break;
-	Cbuf_AddText(cmd);
+	Cbuf_AddText("%s", cmd);
     }
 }
 
@@ -1173,10 +1199,10 @@ SV_CheckVars
 
 ===================
 */
-void
+static void
 SV_CheckVars(void)
 {
-    static char *pw, *spw;
+    static const char *pw, *spw;
     int v;
 
     if (password.string == pw && spectator_password.string == spw)
@@ -1194,8 +1220,7 @@ SV_CheckVars(void)
     if (!v)
 	Info_SetValueForKey(svs.info, "needpass", "", MAX_SERVERINFO_STRING);
     else
-	Info_SetValueForKey(svs.info, "needpass", va("%i", v),
-			    MAX_SERVERINFO_STRING);
+	Info_SetValueForKey(svs.info, "needpass", va("%i", v), MAX_SERVERINFO_STRING);
 }
 
 /*
@@ -1261,19 +1286,20 @@ SV_Frame(float time)
     }
 }
 
-/*
-===============
-SV_InitLocal
-===============
-*/
-void
-SV_InitLocal(void)
+static void
+SV_AddCommands()
 {
-    int i;
+    Cmd_AddCommand("addip", SV_AddIP_f);
+    Cmd_AddCommand("removeip", SV_RemoveIP_f);
+    Cmd_AddCommand("listip", SV_ListIP_f);
+    Cmd_AddCommand("writeip", SV_WriteIP_f);
 
-    SV_InitOperatorCommands();
-    SV_UserInit();
+    SV_AddOperatorCommands();
+}
 
+static void
+SV_RegisterVariables()
+{
     Cvar_RegisterVariable(&rcon_password);
     Cvar_RegisterVariable(&password);
     Cvar_RegisterVariable(&spectator_password);
@@ -1305,7 +1331,6 @@ SV_InitLocal(void)
     Cvar_RegisterVariable(&sv_wateraccelerate);
     Cvar_RegisterVariable(&sv_friction);
     Cvar_RegisterVariable(&sv_waterfriction);
-
     Cvar_RegisterVariable(&sv_aim);
 
     Cvar_RegisterVariable(&filterban);
@@ -1323,20 +1348,23 @@ SV_InitLocal(void)
     Cvar_RegisterVariable(&pausable);
 
     Cvar_RegisterVariable(&developer);
+}
+
+/*
+===============
+SV_InitLocal
+===============
+*/
+static void
+SV_InitLocal(void)
+{
+    SV_ModelInit();
+    SV_UserInit();
+
     if (COM_CheckParm("-developer"))
 	Cvar_SetValue("developer", 1);
 
-    Cmd_AddCommand("addip", SV_AddIP_f);
-    Cmd_AddCommand("removeip", SV_RemoveIP_f);
-    Cmd_AddCommand("listip", SV_ListIP_f);
-    Cmd_AddCommand("writeip", SV_WriteIP_f);
-
-    for (i = 0; i < MAX_MODELS; i++)
-	sprintf(localmodels[i], "*%i", i);
-
-    Info_SetValueForStarKey(svs.info, "*version",
-			    va("TyrQuake-%s", stringify(TYR_VERSION)),
-			    MAX_SERVERINFO_STRING);
+    Info_SetValueForStarKey(svs.info, "*version", va("TyrQuake-%s", build_version), MAX_SERVERINFO_STRING);
 
     // init fraglog stuff
     svs.logsequence = 1;
@@ -1363,7 +1391,7 @@ let it know we are alive, and log information
 ================
 */
 #define	HEARTBEAT_SECONDS	300
-void
+static void
 Master_Heartbeat(void)
 {
     char string[2048];
@@ -1385,7 +1413,7 @@ Master_Heartbeat(void)
 	    active++;
 
     svs.heartbeat_sequence++;
-    sprintf(string, "%c\n%i\n%i\n", S2M_HEARTBEAT,
+    qsnprintf(string, sizeof(string), "%c\n%i\n%i\n", S2M_HEARTBEAT,
 	    svs.heartbeat_sequence, active);
 
     // send to group master
@@ -1404,13 +1432,13 @@ Master_Shutdown
 Informs all masters that this server is going down
 =================
 */
-void
+static void
 Master_Shutdown(void)
 {
     char string[2048];
     int i;
 
-    sprintf(string, "%c\n", S2M_SHUTDOWN);
+    qsnprintf(string, sizeof(string), "%c\n", S2M_SHUTDOWN);
 
     // send to group master
     for (i = 0; i < MAX_MASTERS; i++)
@@ -1443,8 +1471,7 @@ SV_ExtractFromUserinfo(client_t *cl)
     val = Info_ValueForKey(cl->userinfo, "name");
 
     // trim user name
-    strncpy(newname, val, sizeof(newname) - 1);
-    newname[sizeof(newname) - 1] = 0;
+    qstrncpy(newname, val, sizeof(newname));
 
     for (p = newname; (*p == ' ' || *p == '\r' || *p == '\n') && *p; p++);
 
@@ -1491,7 +1518,7 @@ SV_ExtractFromUserinfo(client_t *cl)
 		    p = val + 4;
 	    }
 
-	    sprintf(newname, "(%d)%-.40s", dupc++, p);
+	    qsnprintf(newname, sizeof(newname), "(%d)%-.40s", dupc++, p);
 	    Info_SetValueForKey(cl->userinfo, "name", newname,
 				MAX_INFO_STRING);
 	    val = Info_ValueForKey(cl->userinfo, "name");
@@ -1520,7 +1547,7 @@ SV_ExtractFromUserinfo(client_t *cl)
     }
 
 
-    strncpy(cl->name, val, sizeof(cl->name) - 1);
+    qstrncpy(cl->name, val, sizeof(cl->name));
 
     // rate command
     val = Info_ValueForKey(cl->userinfo, "rate");
@@ -1548,7 +1575,7 @@ SV_ExtractFromUserinfo(client_t *cl)
 SV_InitNet
 ====================
 */
-void
+static void
 SV_InitNet(void)
 {
     int port;
@@ -1564,11 +1591,59 @@ SV_InitNet(void)
 
     Netchan_Init();
 
-    // heartbeats will allways be sent to the id master
+    // heartbeats will always be sent to the id master
     svs.last_heartbeat = -99999;	// send immediately
 //      NET_StringToAdr ("192.246.40.70:27000", &idmaster_adr);
 }
 
+/*
+ * Model Loader Functions
+ */
+
+static alias_loader_t SV_AliasLoader = {
+    NULL, /* Does not load alias models */
+};
+
+static void
+SV_BrushModelLoadLighting(brushmodel_t *brushmodel, dheader_t *header)
+{
+    brushmodel->lightdata = Mod_LoadBytes(brushmodel, header, LUMP_LIGHTING);
+}
+static int SV_BrushModelPadding() { return 0; }
+static void SV_BrushModelPostProcess(brushmodel_t *brushmodel) {}
+static brush_loader_t SV_BrushLoader = {
+    .Padding = SV_BrushModelPadding,
+    .LoadLighting = SV_BrushModelLoadLighting,
+    .PostProcess = SV_BrushModelPostProcess,
+    .lightmap_sample_bytes = 1,
+};
+
+static texture_t qwsv_notexture_mip;
+texture_t *r_notexture_mip = &qwsv_notexture_mip;
+
+//============================================================================
+
+static void
+Commands_Init()
+{
+    Memory_AddCommands();
+    Cmd_AddCommands();
+    COM_AddCommands();
+    Mod_AddCommands();
+    PR_AddCommands();
+    SV_AddCommands();
+}
+
+static void
+Cvars_Init()
+{
+    Sys_RegisterVariables();
+    COM_RegisterVariables();
+    PR_RegisterVariables();
+    SV_RegisterVariables();
+
+    Netchan_RegisterVariables();
+}
 
 /*
 ====================
@@ -1592,28 +1667,28 @@ SV_Init(quakeparms_t *parms)
 		 parms->memsize / (float)0x100000);
 
     Memory_Init(parms->membase, parms->memsize);
-    Cbuf_Init();
-    Cmd_Init();
+    Cvars_Init();
+    Commands_Init();
 
-    COM_Init();
+    Cbuf_Init();
+    COM_InitFileSystem();
+    COM_InitGameDirectoryFromCommandLine();
 
     PR_Init();
-    Mod_Init();
+    Mod_Init(&SV_AliasLoader, &SV_BrushLoader);
 
     SV_InitNet();
 
     SV_InitLocal();
-    Sys_Init();
-    Pmove_Init();
 
     Hunk_AllocName(0, "-HOST_HUNKLEVEL-");
     host_hunklevel = Hunk_LowMark();
 
     host_initialized = true;
 
-    Con_Printf("Exe: " __TIME__ " " __DATE__ "\n");
+    Con_Printf("Exe: %s\n", Build_DateString());
     Con_Printf("%4.1f megabyte heap\n", parms->memsize / (1024 * 1024.0));
-    Con_Printf("\nServer Version TyrQuake-%s\n\n", stringify(TYR_VERSION));
+    Con_Printf("\nServer Version TyrQuake-%s\n\n", build_version);
     Con_Printf("======== QuakeWorld Initialized ========\n");
 
     /*
@@ -1628,3 +1703,13 @@ SV_Init(quakeparms_t *parms)
     if (sv.state == ss_dead)
 	SV_Error("Couldn't spawn a server");
 }
+
+/*
+ * Re-initialise the host after filesystem/gamedir re-init
+ */
+void
+SV_Reinit()
+{
+    Hunk_AllocName(0, "--HOST--");
+    host_hunklevel = Hunk_LowMark();
+}
