@@ -14,11 +14,12 @@ See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
 #include <windows.h>
+#include <mmsystem.h>
 #include <dinput.h>
 
 #include "client.h"
@@ -35,32 +36,50 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "host.h"
 #endif
 
-#define DINPUT_BUFFERSIZE           16
+/* For relative mouse movement and centering */
+static int window_center_x, window_center_y;
+static RECT window_rect;
 
-static HRESULT (WINAPI * pDirectInputCreate) (HINSTANCE hinst, DWORD dwVersion,
-					      LPDIRECTINPUT * lplpDirectInput,
-					      LPUNKNOWN punkOuter);
+unsigned int uiWheelMessage;
+qboolean mouseactive;
+
+static qboolean have_focus;
+
+void
+IN_SetFocus(qboolean focus)
+{
+    have_focus = focus;
+}
+
+qboolean
+IN_HaveFocus()
+{
+    return have_focus;
+}
+
+#define DINPUT_BUFFERSIZE 16
+
+static HRESULT (WINAPI *pDirectInputCreate)(HINSTANCE hinst, DWORD dwVersion,
+					    LPDIRECTINPUT *lplpDirectInput,
+					    LPUNKNOWN punkOuter);
 
 // mouse variables
-cvar_t m_filter = { "m_filter", "0" };
+cvar_t _windowed_mouse = { "_windowed_mouse", "1", CVAR_CONFIG };
+static cvar_t m_filter = { "m_filter", "0" };
 
-int mouse_buttons;
-int mouse_oldbuttonstate;
-POINT current_pos;
-int mouse_x, mouse_y, old_mouse_x, old_mouse_y, mx_accum, my_accum;
+static int mouse_buttons;
+static POINT current_pos;
+static int mouse_x, mouse_y, old_mouse_x, old_mouse_y, mx_accum, my_accum;
+static unsigned int mouse_button_state;
 
 static qboolean restore_spi;
 static int originalmouseparms[3], newmouseparms[3] = { 0, 0, 1 };
 
-unsigned int uiWheelMessage;
-qboolean mouseactive;
-qboolean mouseinitialized;
+static qboolean mouseinitialized;
 static qboolean mouseparmsvalid, mouseactivatetoggle;
 static qboolean mouseshowtoggle = 1;
 static qboolean dinput_acquired;
 
-static unsigned int mstate_di;
-
 // joystick defines and variables
 // where should defines be moved?
 #define JOY_ABSOLUTE_AXIS	0x00000000	// control like a joystick
@@ -77,46 +96,46 @@ enum _ControlList {
     AxisNada = 0, AxisForward, AxisLook, AxisSide, AxisTurn
 };
 
-DWORD dwAxisFlags[JOY_MAX_AXES] = {
+static DWORD dwAxisFlags[JOY_MAX_AXES] = {
     JOY_RETURNX, JOY_RETURNY, JOY_RETURNZ, JOY_RETURNR, JOY_RETURNU,
     JOY_RETURNV
 };
 
-DWORD dwAxisMap[JOY_MAX_AXES];
-DWORD dwControlMap[JOY_MAX_AXES];
-PDWORD pdwRawValue[JOY_MAX_AXES];
+static DWORD dwAxisMap[JOY_MAX_AXES];
+static DWORD dwControlMap[JOY_MAX_AXES];
+static PDWORD pdwRawValue[JOY_MAX_AXES];
 
 // none of these cvars are saved over a session
 // this means that advanced controller configuration needs to be executed
 // each time.  this avoids any problems with getting back to a default usage
 // or when changing from one controller to another.  this way at least something
 // works.
-cvar_t in_joystick = { "joystick", "0", true };
-cvar_t joy_name = { "joyname", "joystick" };
-cvar_t joy_advanced = { "joyadvanced", "0" };
-cvar_t joy_advaxisx = { "joyadvaxisx", "0" };
-cvar_t joy_advaxisy = { "joyadvaxisy", "0" };
-cvar_t joy_advaxisz = { "joyadvaxisz", "0" };
-cvar_t joy_advaxisr = { "joyadvaxisr", "0" };
-cvar_t joy_advaxisu = { "joyadvaxisu", "0" };
-cvar_t joy_advaxisv = { "joyadvaxisv", "0" };
-cvar_t joy_forwardthreshold = { "joyforwardthreshold", "0.15" };
-cvar_t joy_sidethreshold = { "joysidethreshold", "0.15" };
-cvar_t joy_pitchthreshold = { "joypitchthreshold", "0.15" };
-cvar_t joy_yawthreshold = { "joyyawthreshold", "0.15" };
-cvar_t joy_forwardsensitivity = { "joyforwardsensitivity", "-1.0" };
-cvar_t joy_sidesensitivity = { "joysidesensitivity", "-1.0" };
-cvar_t joy_pitchsensitivity = { "joypitchsensitivity", "1.0" };
-cvar_t joy_yawsensitivity = { "joyyawsensitivity", "-1.0" };
-cvar_t joy_wwhack1 = { "joywwhack1", "0.0" };
-cvar_t joy_wwhack2 = { "joywwhack2", "0.0" };
+static cvar_t in_joystick = { "joystick", "0", CVAR_CONFIG };
+static cvar_t joy_name = { "joyname", "joystick" };
+static cvar_t joy_advanced = { "joyadvanced", "0" };
+static cvar_t joy_advaxisx = { "joyadvaxisx", "0" };
+static cvar_t joy_advaxisy = { "joyadvaxisy", "0" };
+static cvar_t joy_advaxisz = { "joyadvaxisz", "0" };
+static cvar_t joy_advaxisr = { "joyadvaxisr", "0" };
+static cvar_t joy_advaxisu = { "joyadvaxisu", "0" };
+static cvar_t joy_advaxisv = { "joyadvaxisv", "0" };
+static cvar_t joy_forwardthreshold = { "joyforwardthreshold", "0.15" };
+static cvar_t joy_sidethreshold = { "joysidethreshold", "0.15" };
+static cvar_t joy_pitchthreshold = { "joypitchthreshold", "0.15" };
+static cvar_t joy_yawthreshold = { "joyyawthreshold", "0.15" };
+static cvar_t joy_forwardsensitivity = { "joyforwardsensitivity", "-1.0" };
+static cvar_t joy_sidesensitivity = { "joysidesensitivity", "-1.0" };
+static cvar_t joy_pitchsensitivity = { "joypitchsensitivity", "1.0" };
+static cvar_t joy_yawsensitivity = { "joyyawsensitivity", "-1.0" };
+static cvar_t joy_wwhack1 = { "joywwhack1", "0.0" };
+static cvar_t joy_wwhack2 = { "joywwhack2", "0.0" };
 
-qboolean joy_avail, joy_advancedinit, joy_haspov;
-DWORD joy_oldbuttonstate, joy_oldpovstate;
+static qboolean joy_avail, joy_advancedinit, joy_haspov;
+static DWORD joy_oldbuttonstate, joy_oldpovstate;
 
-int joy_id;
-DWORD joy_flags;
-DWORD joy_numbuttons;
+static int joy_id;
+static DWORD joy_flags;
+static DWORD joy_numbuttons;
 
 static LPDIRECTINPUT g_pdi;
 static LPDIRECTINPUTDEVICE g_pMouse;
@@ -137,21 +156,20 @@ typedef struct MYDATA {
     BYTE bButtonD;		// Another button goes here
 } MYDATA;
 
+/* FIXME(?): These names may not make sense - just tidying up below */
+#define QAXIS_ANY   (DIDFT_AXIS | DIDFT_ANYINSTANCE)
+#define QAXIS_ALT   (DIDFT_AXIS | DIDFT_ANYINSTANCE | 0x80000000)
+#define QBUTTON_ANY (DIDFT_BUTTON | DIDFT_ANYINSTANCE)
+#define QBUTTON_ALT (DIDFT_BUTTON | DIDFT_ANYINSTANCE | 0x80000000)
+
 static DIOBJECTDATAFORMAT rgodf[] = {
-    {&GUID_XAxis, FIELD_OFFSET(MYDATA, lX), DIDFT_AXIS | DIDFT_ANYINSTANCE,
-     0,},
-    {&GUID_YAxis, FIELD_OFFSET(MYDATA, lY), DIDFT_AXIS | DIDFT_ANYINSTANCE,
-     0,},
-    {&GUID_ZAxis, FIELD_OFFSET(MYDATA, lZ),
-     0x80000000 | DIDFT_AXIS | DIDFT_ANYINSTANCE, 0,},
-    {0, FIELD_OFFSET(MYDATA, bButtonA), DIDFT_BUTTON | DIDFT_ANYINSTANCE,
-     0,},
-    {0, FIELD_OFFSET(MYDATA, bButtonB), DIDFT_BUTTON | DIDFT_ANYINSTANCE,
-     0,},
-    {0, FIELD_OFFSET(MYDATA, bButtonC),
-     0x80000000 | DIDFT_BUTTON | DIDFT_ANYINSTANCE, 0,},
-    {0, FIELD_OFFSET(MYDATA, bButtonD),
-     0x80000000 | DIDFT_BUTTON | DIDFT_ANYINSTANCE, 0,},
+    { &GUID_XAxis, FIELD_OFFSET(MYDATA, lX), QAXIS_ANY, 0 },
+    { &GUID_YAxis, FIELD_OFFSET(MYDATA, lY), QAXIS_ANY, 0 },
+    { &GUID_ZAxis, FIELD_OFFSET(MYDATA, lZ), QAXIS_ALT, 0 },
+    { 0, FIELD_OFFSET(MYDATA, bButtonA), QBUTTON_ANY, 0 },
+    { 0, FIELD_OFFSET(MYDATA, bButtonB), QBUTTON_ANY, 0 },
+    { 0, FIELD_OFFSET(MYDATA, bButtonC), QBUTTON_ALT, 0 },
+    { 0, FIELD_OFFSET(MYDATA, bButtonD), QBUTTON_ALT, 0 },
 };
 
 #define NUM_OBJECTS (sizeof(rgodf) / sizeof(rgodf[0]))
@@ -166,22 +184,38 @@ static DIDATAFORMAT df = {
 };
 
 // forward-referenced functions
-void IN_StartupJoystick(void);
-void Joy_AdvancedUpdate_f(void);
-void IN_JoyMove(usercmd_t *cmd);
-
+static void IN_StartupJoystick(void);
+static void Joy_AdvancedUpdate_f(void);
+static void IN_JoyMove(usercmd_t *cmd);
 
 /*
 ===========
 Force_CenterView_f
 ===========
 */
-void
+static void
 Force_CenterView_f(void)
 {
     cl.viewangles[PITCH] = 0;
 }
 
+/*
+===========
+IN_UpdateWindowRect
+===========
+*/
+void
+IN_UpdateWindowRect(int x, int y, int width, int height)
+{
+    window_rect.left = x;
+    window_rect.top = y;
+    window_rect.right = x + width;
+    window_rect.bottom = y + height;
+    window_center_x = (window_rect.left + window_rect.right) / 2;
+    window_center_y = (window_rect.top + window_rect.bottom) / 2;
+
+    IN_UpdateClipCursor();
+}
 
 /*
 ===========
@@ -204,7 +238,6 @@ IN_ShowMouse
 void
 IN_ShowMouse(void)
 {
-
     if (!mouseshowtoggle) {
 	ShowCursor(TRUE);
 	mouseshowtoggle = 1;
@@ -220,7 +253,6 @@ IN_HideMouse
 void
 IN_HideMouse(void)
 {
-
     if (mouseshowtoggle) {
 	ShowCursor(FALSE);
 	mouseshowtoggle = 0;
@@ -236,7 +268,6 @@ IN_ActivateMouse
 void
 IN_ActivateMouse(void)
 {
-
     mouseactivatetoggle = true;
 
     if (mouseinitialized) {
@@ -251,9 +282,7 @@ IN_ActivateMouse(void)
 	    }
 	} else {
 	    if (mouseparmsvalid)
-		restore_spi =
-		    SystemParametersInfo(SPI_SETMOUSE, 0, newmouseparms, 0);
-
+		restore_spi = SystemParametersInfo(SPI_SETMOUSE, 0, newmouseparms, 0);
 	    SetCursorPos(window_center_x, window_center_y);
 	    SetCapture(mainwindow);
 	    ClipCursor(&window_rect);
@@ -263,20 +292,6 @@ IN_ActivateMouse(void)
     }
 }
 
-
-/*
-===========
-IN_SetQuakeMouseState
-===========
-*/
-void
-IN_SetQuakeMouseState(void)
-{
-    if (mouseactivatetoggle)
-	IN_ActivateMouse();
-}
-
-
 /*
 ===========
 IN_DeactivateMouse
@@ -285,7 +300,6 @@ IN_DeactivateMouse
 void
 IN_DeactivateMouse(void)
 {
-
     mouseactivatetoggle = false;
 
     if (mouseinitialized) {
@@ -308,32 +322,12 @@ IN_DeactivateMouse(void)
     }
 }
 
-
-/*
-===========
-IN_RestoreOriginalMouseState
-===========
-*/
-void
-IN_RestoreOriginalMouseState(void)
-{
-    if (mouseactivatetoggle) {
-	IN_DeactivateMouse();
-	mouseactivatetoggle = true;
-    }
-// try to redraw the cursor so it gets reinitialized, because sometimes it
-// has garbage after the mode switch
-    ShowCursor(TRUE);
-    ShowCursor(FALSE);
-}
-
-
 /*
 ===========
 IN_InitDInput
 ===========
 */
-qboolean
+static qboolean
 IN_InitDInput(void)
 {
     HRESULT hr;
@@ -349,7 +343,6 @@ IN_InitDInput(void)
 
     if (!hInstDI) {
 	hInstDI = LoadLibrary("dinput.dll");
-
 	if (hInstDI == NULL) {
 	    Con_SafePrintf("Couldn't load dinput.dll\n");
 	    return false;
@@ -357,49 +350,44 @@ IN_InitDInput(void)
     }
 
     if (!pDirectInputCreate) {
-	pDirectInputCreate =
-	    (void *)GetProcAddress(hInstDI, "DirectInputCreateA");
-
+	pDirectInputCreate = (void *)GetProcAddress(hInstDI, "DirectInputCreateA");
 	if (!pDirectInputCreate) {
 	    Con_SafePrintf("Couldn't get DI proc addr\n");
 	    return false;
 	}
     }
-// register with DirectInput and get an IDirectInput to play with.
-    hr = pDirectInputCreate(global_hInstance, DIRECTINPUT_VERSION, &g_pdi,
-			    NULL);
 
+    // register with DirectInput and get an IDirectInput to play with.
+    hr = pDirectInputCreate(global_hInstance, DIRECTINPUT_VERSION, &g_pdi, NULL);
     if (FAILED(hr)) {
+	Con_SafePrintf("Unable to create a DI handle\n");
 	return false;
     }
-// obtain an interface to the system mouse device.
-    hr = IDirectInput_CreateDevice(g_pdi, &GUID_SysMouse, &g_pMouse, NULL);
 
+    // obtain an interface to the system mouse device.
+    hr = IDirectInput_CreateDevice(g_pdi, &GUID_SysMouse, &g_pMouse, NULL);
     if (FAILED(hr)) {
 	Con_SafePrintf("Couldn't open DI mouse device\n");
 	return false;
     }
-// set the data format to "mouse format".
-    hr = IDirectInputDevice_SetDataFormat(g_pMouse, &df);
 
+    // set the data format to "mouse format".
+    hr = IDirectInputDevice_SetDataFormat(g_pMouse, &df);
     if (FAILED(hr)) {
 	Con_SafePrintf("Couldn't set DI mouse format\n");
 	return false;
     }
-// set the cooperativity level.
-    hr = IDirectInputDevice_SetCooperativeLevel(g_pMouse, mainwindow,
-						DISCL_EXCLUSIVE |
-						DISCL_FOREGROUND);
 
+    // set the cooperativity level.
+    hr = IDirectInputDevice_SetCooperativeLevel(g_pMouse, mainwindow, DISCL_EXCLUSIVE |	DISCL_FOREGROUND);
     if (FAILED(hr)) {
 	Con_SafePrintf("Couldn't set DI coop level\n");
 	return false;
     }
-// set the buffer size to DINPUT_BUFFERSIZE elements.
-// the buffer size is a DWORD property associated with the device
-    hr = IDirectInputDevice_SetProperty(g_pMouse, DIPROP_BUFFERSIZE,
-					&dipdw.diph);
 
+    // set the buffer size to DINPUT_BUFFERSIZE elements.
+    // the buffer size is a DWORD property associated with the device
+    hr = IDirectInputDevice_SetProperty(g_pMouse, DIPROP_BUFFERSIZE, &dipdw.diph);
     if (FAILED(hr)) {
 	Con_SafePrintf("Couldn't set DI buffersize\n");
 	return false;
@@ -414,7 +402,7 @@ IN_InitDInput(void)
 IN_StartupMouse
 ===========
 */
-void
+static void
 IN_StartupMouse(void)
 {
     if (COM_CheckParm("-nomouse"))
@@ -424,7 +412,6 @@ IN_StartupMouse(void)
 
     if (!COM_CheckParm("-nodinput")) {
 	dinput = IN_InitDInput();
-
 	if (dinput) {
 	    Con_SafePrintf("DirectInput initialized\n");
 	} else {
@@ -433,9 +420,7 @@ IN_StartupMouse(void)
     }
 
     if (!dinput) {
-	mouseparmsvalid =
-	    SystemParametersInfo(SPI_GETMOUSE, 0, originalmouseparms, 0);
-
+	mouseparmsvalid = SystemParametersInfo(SPI_GETMOUSE, 0, originalmouseparms, 0);
 	if (mouseparmsvalid) {
 	    if (COM_CheckParm("-noforcemspd"))
 		newmouseparms[2] = originalmouseparms[2];
@@ -461,17 +446,19 @@ IN_StartupMouse(void)
 	IN_ActivateMouse();
 }
 
-
-/*
-===========
-IN_Init
-===========
-*/
 void
-IN_Init(void)
+IN_AddCommands()
+{
+    Cmd_AddCommand("force_centerview", Force_CenterView_f);
+    Cmd_AddCommand("joyadvancedupdate", Joy_AdvancedUpdate_f);
+}
+
+void
+IN_RegisterVariables()
 {
     // mouse variables
     Cvar_RegisterVariable(&m_filter);
+    Cvar_RegisterVariable(&_windowed_mouse);
 
     // joystick variables
     Cvar_RegisterVariable(&in_joystick);
@@ -493,10 +480,16 @@ IN_Init(void)
     Cvar_RegisterVariable(&joy_yawsensitivity);
     Cvar_RegisterVariable(&joy_wwhack1);
     Cvar_RegisterVariable(&joy_wwhack2);
+}
 
-    Cmd_AddCommand("force_centerview", Force_CenterView_f);
-    Cmd_AddCommand("joyadvancedupdate", Joy_AdvancedUpdate_f);
-
+/*
+===========
+IN_Init
+===========
+*/
+void
+IN_Init(void)
+{
     uiWheelMessage = RegisterWindowMessage("MSWHEEL_ROLLMSG");
 
     IN_StartupMouse();
@@ -511,7 +504,6 @@ IN_Shutdown
 void
 IN_Shutdown(void)
 {
-
     IN_DeactivateMouse();
     IN_ShowMouse();
 
@@ -533,23 +525,19 @@ IN_MouseEvent
 ===========
 */
 void
-IN_MouseEvent(int mstate)
+IN_MouseEvent(int button_state)
 {
     int i;
 
-    if (mouseactive && !dinput) {
+    if (mouseactive) {
 	// perform button actions
 	for (i = 0; i < mouse_buttons; i++) {
-	    if ((mstate & (1 << i)) && !(mouse_oldbuttonstate & (1 << i))) {
+	    if ((button_state & (1 << i)) && !(mouse_button_state & (1 << i)))
 		Key_Event(K_MOUSE1 + i, true);
-	    }
-
-	    if (!(mstate & (1 << i)) && (mouse_oldbuttonstate & (1 << i))) {
+	    if (!(button_state & (1 << i)) && (mouse_button_state & (1 << i)))
 		Key_Event(K_MOUSE1 + i, false);
-	    }
 	}
-
-	mouse_oldbuttonstate = mstate;
+	mouse_button_state = button_state;
     }
 }
 
@@ -563,50 +551,45 @@ void
 IN_MouseMove(usercmd_t *cmd)
 {
     int mx, my;
-    int i;
-    DIDEVICEOBJECTDATA od;
-    DWORD dwElements;
-    HRESULT hr;
+    DIDEVICEOBJECTDATA mouse_state;
+    HRESULT result;
+    unsigned int button_state = 0;
 
     if (!mouseactive)
 	return;
 
     if (dinput) {
+
 	mx = 0;
 	my = 0;
 
 	for (;;) {
-	    dwElements = 1;
-
-	    hr = IDirectInputDevice_GetDeviceData(g_pMouse,
-						  sizeof
-						  (DIDEVICEOBJECTDATA), &od,
-						  &dwElements, 0);
-
-	    if ((hr == DIERR_INPUTLOST) || (hr == DIERR_NOTACQUIRED)) {
-		dinput_acquired = true;
+	    DWORD count = 1;
+	    result = IDirectInputDevice_GetDeviceData(g_pMouse, sizeof(mouse_state), &mouse_state, &count, 0);
+	    if ((result == DIERR_INPUTLOST) || (result == DIERR_NOTACQUIRED)) {
 		IDirectInputDevice_Acquire(g_pMouse);
+		dinput_acquired = true;
 		break;
 	    }
 
 	    /* Unable to read data or no data available */
-	    if (FAILED(hr) || dwElements == 0) {
+	    if (FAILED(result) || count == 0)
 		break;
-	    }
+
+	    /* Ignore mouse events when menu/console is active */
+	    if (key_dest != key_game)
+		continue;
 
 	    /* Look at the element to see what happened */
-
-	    switch (od.dwOfs) {
+	    switch (mouse_state.dwOfs) {
 	    case DIMOFS_X:
-		mx += (int)od.dwData;
+		mx += (int)mouse_state.dwData;
 		break;
-
 	    case DIMOFS_Y:
-		my += (int)od.dwData;
+		my += (int)mouse_state.dwData;
 		break;
-
 	    case DIMOFS_Z:
-		if ((int)od.dwData > 0) {
+		if ((int)mouse_state.dwData > 0) {
 		    Key_Event(K_MWHEELUP, true);
 		    Key_Event(K_MWHEELUP, false);
 		} else {
@@ -614,42 +597,17 @@ IN_MouseMove(usercmd_t *cmd)
 		    Key_Event(K_MWHEELDOWN, false);
 		}
 		break;
-
 	    case DIMOFS_BUTTON0:
-		if (od.dwData & 0x80)
-		    mstate_di |= 1;
-		else
-		    mstate_di &= ~1;
-		break;
-
 	    case DIMOFS_BUTTON1:
-		if (od.dwData & 0x80)
-		    mstate_di |= (1 << 1);
-		else
-		    mstate_di &= ~(1 << 1);
-		break;
-
 	    case DIMOFS_BUTTON2:
-		if (od.dwData & 0x80)
-		    mstate_di |= (1 << 2);
+		if (mouse_state.dwData & 0x80)
+		    button_state |= (1 << (mouse_state.dwOfs - DIMOFS_BUTTON0));
 		else
-		    mstate_di &= ~(1 << 2);
+		    button_state &= ~(1 << (mouse_state.dwOfs - DIMOFS_BUTTON0));
 		break;
 	    }
 	}
-
-	// perform button actions
-	for (i = 0; i < mouse_buttons; i++) {
-	    if ((mstate_di & (1 << i)) && !(mouse_oldbuttonstate & (1 << i))) {
-		Key_Event(K_MOUSE1 + i, true);
-	    }
-
-	    if (!(mstate_di & (1 << i)) && (mouse_oldbuttonstate & (1 << i))) {
-		Key_Event(K_MOUSE1 + i, false);
-	    }
-	}
-
-	mouse_oldbuttonstate = mstate_di;
+	IN_MouseEvent(button_state);
     } else {
 	GetCursorPos(&current_pos);
 	mx = current_pos.x - window_center_x + mx_accum;
@@ -658,6 +616,15 @@ IN_MouseMove(usercmd_t *cmd)
 	my_accum = 0;
     }
 
+    /* If the mouse has moved, force it to the center, so there's room to move */
+    if (mx || my)
+	SetCursorPos(window_center_x, window_center_y);
+
+    /* Ignore mouse events when menu/console is active */
+    if (key_dest != key_game) {
+	return;
+    }
+
     if (m_filter.value) {
 	mouse_x = (mx + old_mouse_x) * 0.5;
 	mouse_y = (my + old_mouse_y) * 0.5;
@@ -673,31 +640,27 @@ IN_MouseMove(usercmd_t *cmd)
     mouse_y *= sensitivity.value;
 
 // add mouse X/Y movement to cmd
-    if ((in_strafe.state & 1) || (lookstrafe.value && (in_mlook.state & 1)))
+    if ((in_strafe.state & 1) || (lookstrafe.value && ((in_mlook.state & 1) ^ (int)m_freelook.value)))
 	cmd->sidemove += m_side.value * mouse_x;
     else
 	cl.viewangles[YAW] -= m_yaw.value * mouse_x;
 
-    if (in_mlook.state & 1)
-	V_StopPitchDrift();
+    if ((in_mlook.state & 1) ^ (int)m_freelook.value)
+	if (mouse_x || mouse_y)
+	    V_StopPitchDrift();
 
-    if ((in_mlook.state & 1) && !(in_strafe.state & 1)) {
+    if (((in_mlook.state & 1) ^ (int)m_freelook.value) && !(in_strafe.state & 1)) {
 	cl.viewangles[PITCH] += m_pitch.value * mouse_y;
-	if (cl.viewangles[PITCH] > 80)
-	    cl.viewangles[PITCH] = 80;
-	if (cl.viewangles[PITCH] < -70)
-	    cl.viewangles[PITCH] = -70;
+	if (cl.viewangles[PITCH] > cl_maxpitch.value)
+	    cl.viewangles[PITCH] = cl_maxpitch.value;
+	if (cl.viewangles[PITCH] < cl_minpitch.value)
+	    cl.viewangles[PITCH] = cl_minpitch.value;
     } else {
 	if ((in_strafe.state & 1) && noclip_anglehack)
 	    cmd->upmove -= m_forward.value * mouse_y;
 	else
 	    cmd->forwardmove -= m_forward.value * mouse_y;
     }
-
-// if the mouse has moved, force it to the center, so there's room to move
-    if (mx || my) {
-	SetCursorPos(window_center_x, window_center_y);
-    }
 }
 
 
@@ -709,8 +672,7 @@ IN_Move
 void
 IN_Move(usercmd_t *cmd)
 {
-
-    if (ActiveApp && window_visible()) {
+    if (have_focus && window_visible()) {
 	IN_MouseMove(cmd);
 	IN_JoyMove(cmd);
     }
@@ -727,9 +689,10 @@ IN_Accumulate(void)
 {
     if (mouseactive && !dinput) {
 	GetCursorPos(&current_pos);
-
-	mx_accum += current_pos.x - window_center_x;
-	my_accum += current_pos.y - window_center_y;
+	if (key_dest == key_game) {
+	    mx_accum += current_pos.x - window_center_x;
+	    my_accum += current_pos.y - window_center_y;
+	}
 
 	/* force the mouse to the center, so there's room to move */
 	SetCursorPos(window_center_x, window_center_y);
@@ -745,11 +708,10 @@ IN_ClearStates
 void
 IN_ClearStates(void)
 {
-
     if (mouseactive) {
 	mx_accum = 0;
 	my_accum = 0;
-	mouse_oldbuttonstate = 0;
+	mouse_button_state = 0;
     }
 }
 
@@ -759,7 +721,7 @@ IN_ClearStates(void)
 IN_StartupJoystick
 ===============
 */
-void
+static void
 IN_StartupJoystick(void)
 {
     int numdevs;
@@ -828,7 +790,7 @@ IN_StartupJoystick(void)
 RawValuePointer
 ===========
 */
-PDWORD
+static PDWORD
 RawValuePointer(int axis)
 {
     switch (axis) {
@@ -855,10 +817,9 @@ RawValuePointer(int axis)
 Joy_AdvancedUpdate_f
 ===========
 */
-void
+static void
 Joy_AdvancedUpdate_f(void)
 {
-
     // called once by IN_ReadJoystick and by user whenever an update is needed
     // cvars are now available
     int i;
@@ -980,10 +941,9 @@ IN_Commands(void)
 IN_ReadJoystick
 ===============
 */
-qboolean
+static qboolean
 IN_ReadJoystick(void)
 {
-
     memset(&ji, 0, sizeof(ji));
     ji.dwSize = sizeof(ji);
     ji.dwFlags = joy_flags;
@@ -1012,7 +972,7 @@ IN_ReadJoystick(void)
 IN_JoyMove
 ===========
 */
-void
+static void
 IN_JoyMove(usercmd_t *cmd)
 {
     float speed, aspeed;
@@ -1034,7 +994,7 @@ IN_JoyMove(usercmd_t *cmd)
 	return;
     }
 
-    if (in_speed.state & 1)
+    if ((in_speed.state & 1) ^ (int)cl_run.value)
 	speed = cl_movespeedkey.value;
     else
 	speed = 1;
@@ -1065,9 +1025,9 @@ IN_JoyMove(usercmd_t *cmd)
 
 	switch (dwAxisMap[i]) {
 	case AxisForward:
-	    if ((joy_advanced.value == 0.0) && (in_mlook.state & 1)) {
+	    if ((joy_advanced.value == 0.0) && ((in_mlook.state & 1) ^ (int)m_freelook.value)) {
 		// user wants forward control to become look control
-		if (fabs(fAxisValue) > joy_pitchthreshold.value) {
+		if (fabsf(fAxisValue) > joy_pitchthreshold.value) {
 		    // if mouse invert is on, invert the joystick pitch value
 		    // only absolute control support here (joy_advanced is false)
 		    if (m_pitch.value < 0.0) {
@@ -1080,17 +1040,10 @@ IN_JoyMove(usercmd_t *cmd)
 			    aspeed * cl_pitchspeed.value;
 		    }
 		    V_StopPitchDrift();
-		} else {
-		    // no pitch movement
-		    // disable pitch return-to-center unless requested by user
-		    // *** this code can be removed when the lookspring bug is fixed
-		    // *** the bug always has the lookspring feature on
-		    if (lookspring.value == 0.0)
-			V_StopPitchDrift();
 		}
 	    } else {
 		// user wants forward control to be forward control
-		if (fabs(fAxisValue) > joy_forwardthreshold.value) {
+		if (fabsf(fAxisValue) > joy_forwardthreshold.value) {
 		    cmd->forwardmove +=
 			(fAxisValue * joy_forwardsensitivity.value) * speed *
 			cl_forwardspeed.value;
@@ -1099,7 +1052,7 @@ IN_JoyMove(usercmd_t *cmd)
 	    break;
 
 	case AxisSide:
-	    if (fabs(fAxisValue) > joy_sidethreshold.value) {
+	    if (fabsf(fAxisValue) > joy_sidethreshold.value) {
 		cmd->sidemove +=
 		    (fAxisValue * joy_sidesensitivity.value) * speed *
 		    cl_sidespeed.value;
@@ -1108,16 +1061,16 @@ IN_JoyMove(usercmd_t *cmd)
 
 	case AxisTurn:
 	    if ((in_strafe.state & 1)
-		|| (lookstrafe.value && (in_mlook.state & 1))) {
+		|| (lookstrafe.value && ((in_mlook.state & 1) ^ (int)m_freelook.value))) {
 		// user wants turn control to become side control
-		if (fabs(fAxisValue) > joy_sidethreshold.value) {
+		if (fabsf(fAxisValue) > joy_sidethreshold.value) {
 		    cmd->sidemove -=
 			(fAxisValue * joy_sidesensitivity.value) * speed *
 			cl_sidespeed.value;
 		}
 	    } else {
 		// user wants turn control to be turn control
-		if (fabs(fAxisValue) > joy_yawthreshold.value) {
+		if (fabsf(fAxisValue) > joy_yawthreshold.value) {
 		    if (dwControlMap[i] == JOY_ABSOLUTE_AXIS) {
 			cl.viewangles[YAW] +=
 			    (fAxisValue * joy_yawsensitivity.value) * aspeed *
@@ -1133,8 +1086,8 @@ IN_JoyMove(usercmd_t *cmd)
 	    break;
 
 	case AxisLook:
-	    if (in_mlook.state & 1) {
-		if (fabs(fAxisValue) > joy_pitchthreshold.value) {
+	    if ((in_mlook.state & 1) ^ (int)m_freelook.value) {
+		if (fabsf(fAxisValue) > joy_pitchthreshold.value) {
 		    // pitch movement detected and pitch movement desired by user
 		    if (dwControlMap[i] == JOY_ABSOLUTE_AXIS) {
 			cl.viewangles[PITCH] +=
@@ -1146,13 +1099,6 @@ IN_JoyMove(usercmd_t *cmd)
 			    speed * 180.0;
 		    }
 		    V_StopPitchDrift();
-		} else {
-		    // no pitch movement
-		    // disable pitch return-to-center unless requested by user
-		    // *** this code can be removed when the lookspring bug is fixed
-		    // *** the bug always has the lookspring feature on
-		    if (lookspring.value == 0.0)
-			V_StopPitchDrift();
 		}
 	    }
 	    break;
